@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* vim: set ts=8 sts=4 et sw=4 tw=99: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -19,6 +19,7 @@
 #include "mozilla/jsipc/CrossProcessObjectWrappers.h"
 #include "mozilla/Likely.h"
 #include "mozilla/dom/ScriptSettings.h"
+#include "mozilla/dom/MaybeCrossOriginObject.h"
 #include "nsContentUtils.h"
 #include "nsXULAppAPI.h"
 
@@ -55,20 +56,26 @@ JSObject* WrapperFactory::GetXrayWaiver(HandleObject obj) {
   XPCWrappedNativeScope* scope = ObjectScope(obj);
   MOZ_ASSERT(scope);
 
-  if (!scope->mWaiverWrapperMap) return nullptr;
+  if (!scope->mWaiverWrapperMap) {
+    return nullptr;
+  }
 
   return scope->mWaiverWrapperMap->Find(obj);
 }
 
-JSObject* WrapperFactory::CreateXrayWaiver(JSContext* cx, HandleObject obj) {
-  // The caller is required to have already done a lookup.
+JSObject* WrapperFactory::CreateXrayWaiver(JSContext* cx, HandleObject obj,
+                                           bool allowExisting) {
+  // The caller is required to have already done a lookup, unless it's
+  // trying to replace an existing waiver.
   // NB: This implictly performs the assertions of GetXrayWaiver.
-  MOZ_ASSERT(!GetXrayWaiver(obj));
+  MOZ_ASSERT(bool(GetXrayWaiver(obj)) == allowExisting);
   XPCWrappedNativeScope* scope = ObjectScope(obj);
 
-  JSAutoCompartment ac(cx, obj);
+  JSAutoRealm ar(cx, obj);
   JSObject* waiver = Wrapper::New(cx, obj, &XrayWaiver);
-  if (!waiver) return nullptr;
+  if (!waiver) {
+    return nullptr;
+  }
 
   // Add the new waiver to the map. It's important that we only ever have
   // one waiver for the lifetime of the target object.
@@ -76,7 +83,9 @@ JSObject* WrapperFactory::CreateXrayWaiver(JSContext* cx, HandleObject obj) {
     scope->mWaiverWrapperMap =
         JSObject2JSObjectMap::newMap(XPC_WRAPPER_MAP_LENGTH);
   }
-  if (!scope->mWaiverWrapperMap->Add(cx, obj, waiver)) return nullptr;
+  if (!scope->mWaiverWrapperMap->Add(cx, obj, waiver)) {
+    return nullptr;
+  }
   return waiver;
 }
 
@@ -89,17 +98,19 @@ JSObject* WrapperFactory::WaiveXray(JSContext* cx, JSObject* objArg) {
   if (!waiver) {
     waiver = CreateXrayWaiver(cx, obj);
   }
-  MOZ_ASSERT(JS::ObjectIsNotGray(waiver));
+  JS::AssertObjectIsNotGray(waiver);
   return waiver;
 }
 
-/* static */ bool WrapperFactory::AllowWaiver(JSCompartment* target,
-                                              JSCompartment* origin) {
+/* static */
+bool WrapperFactory::AllowWaiver(JS::Compartment* target,
+                                 JS::Compartment* origin) {
   return CompartmentPrivate::Get(target)->allowWaivers &&
-         AccessCheck::subsumes(target, origin);
+         CompartmentOriginInfo::Subsumes(target, origin);
 }
 
-/* static */ bool WrapperFactory::AllowWaiver(JSObject* wrapper) {
+/* static */
+bool WrapperFactory::AllowWaiver(JSObject* wrapper) {
   MOZ_ASSERT(js::IsCrossCompartmentWrapper(wrapper));
   return AllowWaiver(js::GetObjectCompartment(wrapper),
                      js::GetObjectCompartment(js::UncheckedUnwrap(wrapper)));
@@ -111,29 +122,32 @@ inline bool ShouldWaiveXray(JSContext* cx, JSObject* originalObj) {
                             &flags);
 
   // If the original object did not point through an Xray waiver, we're done.
-  if (!(flags & WrapperFactory::WAIVE_XRAY_WRAPPER_FLAG)) return false;
+  if (!(flags & WrapperFactory::WAIVE_XRAY_WRAPPER_FLAG)) {
+    return false;
+  }
 
   // If the original object was not a cross-compartment wrapper, that means
   // that the caller explicitly created a waiver. Preserve it so that things
   // like WaiveXrayAndWrap work.
-  if (!(flags & Wrapper::CROSS_COMPARTMENT)) return true;
+  if (!(flags & Wrapper::CROSS_COMPARTMENT)) {
+    return true;
+  }
 
   // Otherwise, this is a case of explicitly passing a wrapper across a
   // compartment boundary. In that case, we only want to preserve waivers
   // in transactions between same-origin compartments.
-  JSCompartment* oldCompartment = js::GetObjectCompartment(originalObj);
-  JSCompartment* newCompartment = js::GetContextCompartment(cx);
+  JS::Compartment* oldCompartment = js::GetObjectCompartment(originalObj);
+  JS::Compartment* newCompartment = js::GetContextCompartment(cx);
   bool sameOrigin = false;
   if (OriginAttributes::IsRestrictOpenerAccessForFPI()) {
     sameOrigin =
-        AccessCheck::subsumesConsideringDomain(oldCompartment,
-                                               newCompartment) &&
-        AccessCheck::subsumesConsideringDomain(newCompartment, oldCompartment);
+        CompartmentOriginInfo::Subsumes(oldCompartment, newCompartment) &&
+        CompartmentOriginInfo::Subsumes(newCompartment, oldCompartment);
   } else {
-    sameOrigin = AccessCheck::subsumesConsideringDomainIgnoringFPD(
-                     oldCompartment, newCompartment) &&
-                 AccessCheck::subsumesConsideringDomainIgnoringFPD(
-                     newCompartment, oldCompartment);
+    sameOrigin = CompartmentOriginInfo::SubsumesIgnoringFPD(oldCompartment,
+                                                            newCompartment) &&
+                 CompartmentOriginInfo::SubsumesIgnoringFPD(newCompartment,
+                                                            oldCompartment);
   }
   return sameOrigin;
 }
@@ -142,51 +156,13 @@ void WrapperFactory::PrepareForWrapping(JSContext* cx, HandleObject scope,
                                         HandleObject objArg,
                                         HandleObject objectPassedToWrap,
                                         MutableHandleObject retObj) {
+  // The JS engine calls ToWindowProxyIfWindow and deals with dead wrappers.
+  MOZ_ASSERT(!js::IsWindow(objArg));
+  MOZ_ASSERT(!JS_IsDeadWrapper(objArg));
+
   bool waive = ShouldWaiveXray(cx, objectPassedToWrap);
   RootedObject obj(cx, objArg);
   retObj.set(nullptr);
-  // Outerize any raw inner objects at the entry point here, so that we don't
-  // have to worry about them for the rest of the wrapping code.
-  if (js::IsWindow(obj)) {
-    obj = js::ToWindowProxyIfWindow(obj);
-    MOZ_ASSERT(obj);
-    // ToWindowProxyIfWindow can return a CCW if |obj| was a
-    // navigated-away-from Window. Strip any CCWs.
-    obj = js::UncheckedUnwrap(obj);
-    if (JS_IsDeadWrapper(obj)) {
-      retObj.set(JS_NewDeadWrapper(cx, obj));
-      return;
-    }
-    MOZ_ASSERT(js::IsWindowProxy(obj));
-    // We crossed a compartment boundary there, so may now have a gray
-    // object.  This function is not allowed to return gray objects, so
-    // don't do that.
-    ExposeObjectToActiveJS(obj);
-  }
-
-  // If the object is a dead wrapper, return a new dead wrapper rather than
-  // trying to wrap it for a different compartment.
-  if (JS_IsDeadWrapper(obj)) {
-    retObj.set(JS_NewDeadWrapper(cx, obj));
-    return;
-  }
-
-  // If we've somehow gotten to this point after either the source or target
-  // compartment has been nuked, return a DeadObjectProxy to prevent further
-  // access.
-  // However, we always need to provide live wrappers for ScriptSourceObjects,
-  // since they're used for cross-compartment cloned scripts, and need to
-  // remain accessible even after the original compartment has been nuked.
-  JSCompartment* origin = js::GetObjectCompartment(obj);
-  JSCompartment* target = js::GetObjectCompartment(scope);
-  if (!JS_IsScriptSourceObject(obj) &&
-      (CompartmentPrivate::Get(origin)->wasNuked ||
-       CompartmentPrivate::Get(target)->wasNuked)) {
-    NS_WARNING("Trying to create a wrapper into or out of a nuked compartment");
-
-    retObj.set(JS_NewDeadWrapper(cx));
-    return;
-  }
 
   // If we've got a WindowProxy, there's nothing special that needs to be
   // done here, and we can move on to the next phase of wrapping. We handle
@@ -212,7 +188,7 @@ void WrapperFactory::PrepareForWrapping(JSContext* cx, HandleObject scope,
 
   XPCWrappedNative* wn = XPCWrappedNative::Get(obj);
 
-  JSAutoCompartment ac(cx, obj);
+  JSAutoRealm ar(cx, obj);
   XPCCallContext ccx(cx, obj);
   RootedObject wrapScope(cx, scope);
 
@@ -242,7 +218,7 @@ void WrapperFactory::PrepareForWrapping(JSContext* cx, HandleObject scope,
         return;
       }
 
-      RootedObject currentScope(cx, JS_GetGlobalForObject(cx, obj));
+      RootedObject currentScope(cx, JS::GetNonCCWObjectGlobal(obj));
       if (MOZ_UNLIKELY(wrapScope != currentScope)) {
         // The wrapper claims it wants to be in the new scope, but
         // currently has a reflection that lives in the old scope. This
@@ -292,8 +268,8 @@ void WrapperFactory::PrepareForWrapping(JSContext* cx, HandleObject scope,
       // compute the correct (opaque) wrapper for the object below given the
       // security characteristics of the two compartments.
       if (!AccessCheck::isChrome(js::GetObjectCompartment(wrapScope)) &&
-          AccessCheck::subsumes(js::GetObjectCompartment(wrapScope),
-                                js::GetObjectCompartment(obj))) {
+          CompartmentOriginInfo::Subsumes(js::GetObjectCompartment(wrapScope),
+                                          js::GetObjectCompartment(obj))) {
         retObj.set(waive ? WaiveXray(cx, obj) : obj);
         return;
       }
@@ -312,7 +288,7 @@ void WrapperFactory::PrepareForWrapping(JSContext* cx, HandleObject scope,
 
   obj.set(&v.toObject());
   MOZ_ASSERT(IS_WN_REFLECTOR(obj), "bad object");
-  MOZ_ASSERT(JS::ObjectIsNotGray(obj), "Should never return gray reflectors");
+  JS::AssertObjectIsNotGray(obj);  // We should never return gray reflectors.
 
   // Because the underlying native didn't have a PreCreate hook, we had
   // to a new (or possibly pre-existing) XPCWN in our compartment.
@@ -325,7 +301,7 @@ void WrapperFactory::PrepareForWrapping(JSContext* cx, HandleObject scope,
   // to do this cleverly in the common case to avoid too much overhead.
   XPCWrappedNative* newwn = XPCWrappedNative::Get(obj);
   RefPtr<XPCNativeSet> unionSet =
-      XPCNativeSet::GetNewOrUsed(newwn->GetSet(), wn->GetSet(), false);
+      XPCNativeSet::GetNewOrUsed(cx, newwn->GetSet(), wn->GetSet(), false);
   if (!unionSet) {
     return;
   }
@@ -334,42 +310,101 @@ void WrapperFactory::PrepareForWrapping(JSContext* cx, HandleObject scope,
   retObj.set(waive ? WaiveXray(cx, obj) : obj);
 }
 
+// This check is completely symmetric, so we don't need to keep track of origin
+// vs target here.  Two compartments may have had transparent CCWs between them
+// only if they are same-origin (ignoring document.domain) or have both had
+// document.domain set at some point and are same-site.  In either case they
+// will have the same SiteIdentifier, so check that first.
+static bool CompartmentsMayHaveHadTransparentCCWs(
+    CompartmentPrivate* private1, CompartmentPrivate* private2) {
+  auto& info1 = private1->originInfo;
+  auto& info2 = private2->originInfo;
+
+  if (!info1.SiteRef().Equals(info2.SiteRef())) {
+    return false;
+  }
+
+  return info1.GetPrincipalIgnoringDocumentDomain()->FastEquals(
+             info2.GetPrincipalIgnoringDocumentDomain()) ||
+         (info1.HasChangedDocumentDomain() && info2.HasChangedDocumentDomain());
+}
+
 #ifdef DEBUG
 static void DEBUG_CheckUnwrapSafety(HandleObject obj,
                                     const js::Wrapper* handler,
-                                    JSCompartment* origin,
-                                    JSCompartment* target) {
-  if (!JS_IsScriptSourceObject(obj) &&
-      (CompartmentPrivate::Get(origin)->wasNuked ||
-       CompartmentPrivate::Get(target)->wasNuked)) {
-    // If either compartment has already been nuked, we should have returned
-    // a dead wrapper from our prewrap callback, and this function should
-    // not be called.
+                                    JS::Realm* origin, JS::Realm* target) {
+  JS::Compartment* targetCompartment = JS::GetCompartmentForRealm(target);
+  if (!js::AllowNewWrapper(targetCompartment, obj)) {
+    // The JS engine should have returned a dead wrapper in this case and we
+    // shouldn't even get here.
     MOZ_ASSERT_UNREACHABLE("CheckUnwrapSafety called for a dead wrapper");
-  } else if (AccessCheck::isChrome(target) ||
-             xpc::IsUniversalXPConnectEnabled(target)) {
+  } else if (AccessCheck::isChrome(targetCompartment) ||
+             xpc::IsUniversalXPConnectEnabled(targetCompartment)) {
     // If the caller is chrome (or effectively so), unwrap should always be
-    // allowed.
-    MOZ_ASSERT(!handler->hasSecurityPolicy());
-  } else if (CompartmentPrivate::Get(origin)->forcePermissiveCOWs) {
+    // allowed, but we might have a CrossOriginObjectWrapper here which allows
+    // it dynamically.
+    MOZ_ASSERT(!handler->hasSecurityPolicy() ||
+               handler == &CrossOriginObjectWrapper::singleton);
+  } else if (RealmPrivate::Get(origin)->forcePermissiveCOWs) {
     // Similarly, if this is a privileged scope that has opted to make itself
     // accessible to the world (allowed only during automation), unwrap should
-    // be allowed.
-    MOZ_ASSERT(!handler->hasSecurityPolicy());
+    // be allowed.  Again, it might be allowed dynamically.
+    MOZ_ASSERT(!handler->hasSecurityPolicy() ||
+               handler == &CrossOriginObjectWrapper::singleton);
   } else {
     // Otherwise, it should depend on whether the target subsumes the origin.
-    MOZ_ASSERT(handler->hasSecurityPolicy() ==
-               !(OriginAttributes::IsRestrictOpenerAccessForFPI()
-                     ? AccessCheck::subsumesConsideringDomain(target, origin)
-                     : AccessCheck::subsumesConsideringDomainIgnoringFPD(
-                           target, origin)));
+    bool subsumes =
+        (OriginAttributes::IsRestrictOpenerAccessForFPI()
+             ? AccessCheck::subsumesConsideringDomain(target, origin)
+             : AccessCheck::subsumesConsideringDomainIgnoringFPD(target,
+                                                                 origin));
+    if (!subsumes) {
+      // If the target (which is where the wrapper lives) does not subsume the
+      // origin (which is where the wrapped object lives), then we should
+      // generally have a security check on the wrapper here.  There is one
+      // exception, though: things that used to be same-origin and then stopped
+      // due to document.domain changes.  In that case we will have a
+      // transparent cross-compartment wrapper here even though "subsumes" is no
+      // longer true.
+      CompartmentPrivate* originCompartmentPrivate =
+          CompartmentPrivate::Get(origin);
+      CompartmentPrivate* targetCompartmentPrivate =
+          CompartmentPrivate::Get(target);
+      if (!originCompartmentPrivate->wantXrays &&
+          !targetCompartmentPrivate->wantXrays &&
+          CompartmentsMayHaveHadTransparentCCWs(originCompartmentPrivate,
+                                                targetCompartmentPrivate)) {
+        // We should have a transparent CCW, unless we have a cross-origin
+        // object, in which case it will be a CrossOriginObjectWrapper.
+        MOZ_ASSERT(handler == &CrossCompartmentWrapper::singleton ||
+                   handler == &CrossOriginObjectWrapper::singleton);
+      } else {
+        MOZ_ASSERT(handler->hasSecurityPolicy());
+      }
+    } else {
+      // Even if target subsumes origin, we might have a wrapper with a security
+      // policy here, if it happens to be a CrossOriginObjectWrapper.
+      MOZ_ASSERT(!handler->hasSecurityPolicy() ||
+                 handler == &CrossOriginObjectWrapper::singleton);
+    }
   }
 }
 #else
-#define DEBUG_CheckUnwrapSafety(obj, handler, origin, target) \
-  {}
+#  define DEBUG_CheckUnwrapSafety(obj, handler, origin, target) \
+    {}
 #endif
 
+const CrossOriginObjectWrapper CrossOriginObjectWrapper::singleton;
+
+bool CrossOriginObjectWrapper::dynamicCheckedUnwrapAllowed(
+    HandleObject obj, JSContext* cx) const {
+  MOZ_ASSERT(js::GetProxyHandler(obj) == this,
+             "Why are we getting called for some random object?");
+  JSObject* target = wrappedObject(obj);
+  return dom::MaybeCrossOriginObjectMixins::IsPlatformObjectSameOrigin(cx,
+                                                                       target);
+}
+
 static const Wrapper* SelectWrapper(bool securityWrapper, XrayType xrayType,
                                     bool waiveXrays, JSObject* obj) {
   // Waived Xray uses a modified CCW that has transparent behavior but
@@ -382,7 +417,9 @@ static const Wrapper* SelectWrapper(bool securityWrapper, XrayType xrayType,
   // If we don't want or can't use Xrays, select a wrapper that's either
   // entirely transparent or entirely opaque.
   if (xrayType == NotXray) {
-    if (!securityWrapper) return &CrossCompartmentWrapper::singleton;
+    if (!securityWrapper) {
+      return &CrossCompartmentWrapper::singleton;
+    }
     return &FilteringWrapper<CrossCompartmentSecurityWrapper,
                              Opaque>::singleton;
   }
@@ -390,22 +427,15 @@ static const Wrapper* SelectWrapper(bool securityWrapper, XrayType xrayType,
   // Ok, we're using Xray. If this isn't a security wrapper, use the permissive
   // version and skip the filter.
   if (!securityWrapper) {
-    if (xrayType == XrayForWrappedNative)
-      return &PermissiveXrayXPCWN::singleton;
-    else if (xrayType == XrayForDOMObject)
+    if (xrayType == XrayForDOMObject) {
       return &PermissiveXrayDOM::singleton;
-    else if (xrayType == XrayForJSObject)
+    } else if (xrayType == XrayForJSObject) {
       return &PermissiveXrayJS::singleton;
+    }
     MOZ_ASSERT(xrayType == XrayForOpaqueObject);
     return &PermissiveXrayOpaque::singleton;
   }
 
-  // This is a security wrapper. Use the security versions and filter.
-  if (xrayType == XrayForDOMObject &&
-      IdentifyCrossOriginObject(obj) != CrossOriginOpaque)
-    return &FilteringWrapper<CrossOriginXrayWrapper,
-                             CrossOriginAccessiblePropertiesOnly>::singleton;
-
   // There's never any reason to expose other objects to non-subsuming actors.
   // Just use an opaque wrapper in these cases.
   //
@@ -414,9 +444,10 @@ static const Wrapper* SelectWrapper(bool securityWrapper, XrayType xrayType,
   // functions exposed from the XBL scope. We could remove this exception,
   // if needed, by using ExportFunction to generate the content-side
   // representations of XBL methods.
-  if (xrayType == XrayForJSObject && IsInContentXBLScope(obj))
+  if (xrayType == XrayForJSObject && IsInContentXBLScope(obj)) {
     return &FilteringWrapper<CrossCompartmentSecurityWrapper,
                              OpaqueWithCall>::singleton;
+  }
   return &FilteringWrapper<CrossCompartmentSecurityWrapper, Opaque>::singleton;
 }
 
@@ -425,14 +456,13 @@ JSObject* WrapperFactory::Rewrap(JSContext* cx, HandleObject existing,
   MOZ_ASSERT(!IsWrapper(obj) || GetProxyHandler(obj) == &XrayWaiver ||
                  js::IsWindowProxy(obj),
              "wrapped object passed to rewrap");
-  MOZ_ASSERT(!XrayUtils::IsXPCWNHolderClass(JS_GetClass(obj)),
-             "trying to wrap a holder");
   MOZ_ASSERT(!js::IsWindow(obj));
   MOZ_ASSERT(dom::IsJSAPIActive());
 
   // Compute the information we need to select the right wrapper.
-  JSCompartment* origin = js::GetObjectCompartment(obj);
-  JSCompartment* target = js::GetContextCompartment(cx);
+  JS::Realm* origin = js::GetNonCCWObjectRealm(obj);
+  JS::Realm* target = js::GetContextRealm(cx);
+  MOZ_ASSERT(target, "Why is our JSContext not in a Realm?");
   bool originIsChrome = AccessCheck::isChrome(origin);
   bool targetIsChrome = AccessCheck::isChrome(target);
   bool originSubsumesTarget =
@@ -452,6 +482,12 @@ JSObject* WrapperFactory::Rewrap(JSContext* cx, HandleObject existing,
   CompartmentPrivate* targetCompartmentPrivate =
       CompartmentPrivate::Get(target);
 
+  RealmPrivate* originRealmPrivate = RealmPrivate::Get(origin);
+
+  // Track whether we decided to use a transparent wrapper because of
+  // document.domain usage, so we don't override that decision.
+  bool isTransparentWrapperDueToDocumentDomain = false;
+
   //
   // First, handle the special cases.
   //
@@ -464,7 +500,7 @@ JSObject* WrapperFactory::Rewrap(JSContext* cx, HandleObject existing,
   }
 
   // Let the SpecialPowers scope make its stuff easily accessible to content.
-  else if (originCompartmentPrivate->forcePermissiveCOWs) {
+  else if (originRealmPrivate->forcePermissiveCOWs) {
     CrashIfNotInAutomation();
     wrapper = &CrossCompartmentWrapper::singleton;
   }
@@ -495,6 +531,31 @@ JSObject* WrapperFactory::Rewrap(JSContext* cx, HandleObject existing,
     }
   }
 
+  // Special handling for the web's cross-origin objects (WindowProxy and
+  // Location).  We only need or want to do this in web-like contexts, where all
+  // security relationships are symmetric and there are no forced Xrays.
+  else if (originSubsumesTarget == targetSubsumesOrigin &&
+           // Check for the more rare case of cross-origin objects before doing
+           // the more-likely-to-pass checks for wantXrays.
+           IsCrossOriginAccessibleObject(obj) &&
+           (!targetSubsumesOrigin || (!originCompartmentPrivate->wantXrays &&
+                                      !targetCompartmentPrivate->wantXrays))) {
+    wrapper = &CrossOriginObjectWrapper::singleton;
+  }
+
+  // Special handling for other web objects.  Again, we only want this in
+  // web-like contexts (symmetric security relationships, no forced Xrays).  In
+  // this situation, if the two compartments may ever have had transparent CCWs
+  // between them, we want to keep using transparent CCWs.
+  else if (originSubsumesTarget == targetSubsumesOrigin &&
+           !originCompartmentPrivate->wantXrays &&
+           !targetCompartmentPrivate->wantXrays &&
+           CompartmentsMayHaveHadTransparentCCWs(originCompartmentPrivate,
+                                                 targetCompartmentPrivate)) {
+    isTransparentWrapperDueToDocumentDomain = true;
+    wrapper = &CrossCompartmentWrapper::singleton;
+  }
+
   //
   // Now, handle the regular cases.
   //
@@ -527,7 +588,8 @@ JSObject* WrapperFactory::Rewrap(JSContext* cx, HandleObject existing,
     wrapper = SelectWrapper(securityWrapper, xrayType, waiveXrays, obj);
   }
 
-  if (!targetSubsumesOrigin && !originCompartmentPrivate->forcePermissiveCOWs) {
+  if (!targetSubsumesOrigin && !originRealmPrivate->forcePermissiveCOWs &&
+      !isTransparentWrapperDueToDocumentDomain) {
     // Do a belt-and-suspenders check against exposing eval()/Function() to
     // non-subsuming content.  But don't worry about doing it in the
     // SpecialPowers case.
@@ -544,7 +606,9 @@ JSObject* WrapperFactory::Rewrap(JSContext* cx, HandleObject existing,
 
   DEBUG_CheckUnwrapSafety(obj, wrapper, origin, target);
 
-  if (existing) return Wrapper::Renew(existing, obj, wrapper);
+  if (existing) {
+    return Wrapper::Renew(existing, obj, wrapper);
+  }
 
   return Wrapper::New(cx, obj, wrapper);
 }
@@ -554,10 +618,14 @@ JSObject* WrapperFactory::Rewrap(JSContext* cx, HandleObject existing,
 // using the returned object. If the object to be wrapped is already in the
 // correct compartment, then this returns the unwrapped object.
 bool WrapperFactory::WaiveXrayAndWrap(JSContext* cx, MutableHandleValue vp) {
-  if (vp.isPrimitive()) return JS_WrapValue(cx, vp);
+  if (vp.isPrimitive()) {
+    return JS_WrapValue(cx, vp);
+  }
 
   RootedObject obj(cx, &vp.toObject());
-  if (!WaiveXrayAndWrap(cx, &obj)) return false;
+  if (!WaiveXrayAndWrap(cx, &obj)) {
+    return false;
+  }
 
   vp.setObject(*obj);
   return true;
@@ -581,12 +649,16 @@ bool WrapperFactory::WaiveXrayAndWrap(JSContext* cx,
   // with the function. So if we find ourselves trying to create a waiver for
   // |cx|, we should check whether the caller has any business with waivers
   // to things in |obj|'s compartment.
-  JSCompartment* target = js::GetContextCompartment(cx);
-  JSCompartment* origin = js::GetObjectCompartment(obj);
+  JS::Compartment* target = js::GetContextCompartment(cx);
+  JS::Compartment* origin = js::GetObjectCompartment(obj);
   obj = AllowWaiver(target, origin) ? WaiveXray(cx, obj) : obj;
-  if (!obj) return false;
+  if (!obj) {
+    return false;
+  }
 
-  if (!JS_WrapObject(cx, &obj)) return false;
+  if (!JS_WrapObject(cx, &obj)) {
+    return false;
+  }
   argObj.set(obj);
   return true;
 }
@@ -597,38 +669,94 @@ bool WrapperFactory::WaiveXrayAndWrap(JSContext* cx,
  */
 
 static bool FixWaiverAfterTransplant(JSContext* cx, HandleObject oldWaiver,
-                                     HandleObject newobj) {
+                                     HandleObject newobj,
+                                     bool crossCompartmentTransplant) {
   MOZ_ASSERT(Wrapper::wrapperHandler(oldWaiver) == &XrayWaiver);
   MOZ_ASSERT(!js::IsCrossCompartmentWrapper(newobj));
 
-  // Create a waiver in the new compartment. We know there's not one already
-  // because we _just_ transplanted, which means that |newobj| was either
-  // created from scratch, or was previously cross-compartment wrapper (which
-  // should have no waiver). CreateXrayWaiver asserts this.
-  JSObject* newWaiver = WrapperFactory::CreateXrayWaiver(cx, newobj);
-  if (!newWaiver) return false;
+  if (crossCompartmentTransplant) {
+    // If the new compartment has a CCW for oldWaiver, nuke this CCW. This
+    // prevents confusing RemapAllWrappersForObject: it would call RemapWrapper
+    // with two same-compartment objects (the CCW and the new waiver).
+    //
+    // This can happen when loading a chrome page in a content frame and there
+    // exists a CCW from the chrome compartment to oldWaiver wrapping the window
+    // we just transplanted:
+    //
+    // Compartment 1  |  Compartment 2
+    // ----------------------------------------
+    // CCW1 -----------> oldWaiver --> CCW2 --+
+    // newWaiver                              |
+    // WindowProxy <--------------------------+
+    js::NukeCrossCompartmentWrapperIfExists(
+        cx, js::GetObjectCompartment(newobj), oldWaiver);
+  } else {
+    // We kept the same object identity, so the waiver should be a
+    // waiver for our object, just in the wrong Realm.
+    MOZ_ASSERT(newobj == Wrapper::wrappedObject(oldWaiver));
+  }
+
+  // Create a waiver in the new compartment. We know there's not one already in
+  // the crossCompartmentTransplant case because we _just_ transplanted, which
+  // means that |newobj| was either created from scratch, or was previously
+  // cross-compartment wrapper (which should have no waiver). On the other hand,
+  // in the !crossCompartmentTransplant case we know one already exists.
+  // CreateXrayWaiver asserts all this.
+  JSObject* newWaiver = WrapperFactory::CreateXrayWaiver(
+      cx, newobj, /* allowExisting = */ !crossCompartmentTransplant);
+  if (!newWaiver) {
+    return false;
+  }
+
+  if (!crossCompartmentTransplant) {
+    // CreateXrayWaiver should have updated the map to point to the new waiver.
+    MOZ_ASSERT(WrapperFactory::GetXrayWaiver(newobj) == newWaiver);
+  }
 
   // Update all the cross-compartment references to oldWaiver to point to
   // newWaiver.
-  if (!js::RemapAllWrappersForObject(cx, oldWaiver, newWaiver)) return false;
+  if (!js::RemapAllWrappersForObject(cx, oldWaiver, newWaiver)) {
+    return false;
+  }
+
+  if (crossCompartmentTransplant) {
+    // There should be no same-compartment references to oldWaiver, and we
+    // just remapped all cross-compartment references. It's dead, so we can
+    // remove it from the map.
+    XPCWrappedNativeScope* scope = ObjectScope(oldWaiver);
+    JSObject* key = Wrapper::wrappedObject(oldWaiver);
+    MOZ_ASSERT(scope->mWaiverWrapperMap->Find(key));
+    scope->mWaiverWrapperMap->Remove(key);
+  }
 
-  // There should be no same-compartment references to oldWaiver, and we
-  // just remapped all cross-compartment references. It's dead, so we can
-  // remove it from the map.
-  XPCWrappedNativeScope* scope = ObjectScope(oldWaiver);
-  JSObject* key = Wrapper::wrappedObject(oldWaiver);
-  MOZ_ASSERT(scope->mWaiverWrapperMap->Find(key));
-  scope->mWaiverWrapperMap->Remove(key);
   return true;
 }
 
 JSObject* TransplantObject(JSContext* cx, JS::HandleObject origobj,
                            JS::HandleObject target) {
   RootedObject oldWaiver(cx, WrapperFactory::GetXrayWaiver(origobj));
+  MOZ_ASSERT_IF(oldWaiver, GetNonCCWObjectRealm(oldWaiver) ==
+                               GetNonCCWObjectRealm(origobj));
   RootedObject newIdentity(cx, JS_TransplantObject(cx, origobj, target));
-  if (!newIdentity || !oldWaiver) return newIdentity;
+  if (!newIdentity || !oldWaiver) {
+    return newIdentity;
+  }
 
-  if (!FixWaiverAfterTransplant(cx, oldWaiver, newIdentity)) return nullptr;
+  bool crossCompartmentTransplant = (newIdentity != origobj);
+  if (!crossCompartmentTransplant) {
+    // We might still have been transplanted across realms within a single
+    // compartment.
+    if (GetNonCCWObjectRealm(oldWaiver) == GetNonCCWObjectRealm(newIdentity)) {
+      // The old waiver is same-realm with the new object; nothing else to do
+      // here.
+      return newIdentity;
+    }
+  }
+
+  if (!FixWaiverAfterTransplant(cx, oldWaiver, newIdentity,
+                                crossCompartmentTransplant)) {
+    return nullptr;
+  }
   return newIdentity;
 }
 
@@ -656,7 +784,7 @@ JSObject* TransplantObjectRetainingXrayExpandos(JSContext* cx,
 }
 
 nsIGlobalObject* NativeGlobal(JSObject* obj) {
-  obj = js::GetGlobalForObjectCrossCompartment(obj);
+  obj = JS::GetNonCCWObjectGlobal(obj);
 
   // Every global needs to hold a native as its private or be a
   // WebIDL object with an nsISupports DOM object.
@@ -685,4 +813,8 @@ nsIGlobalObject* NativeGlobal(JSObject* obj) {
   return global;
 }
 
+nsIGlobalObject* CurrentNativeGlobal(JSContext* cx) {
+  return xpc::NativeGlobal(JS::CurrentGlobalOrNull(cx));
+}
+
 }  // namespace xpc
