@@ -3,22 +3,16 @@
 // This test checks whether applied WebExtension themes are persisted and applied
 // on newly opened windows.
 
-add_task(async function setup() {
-  await SpecialPowers.pushPrefEnv({
-    set: [["extensions.webextensions.themes.enabled", true]],
-  });
-});
-
 add_task(async function test_multiple_windows() {
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
-      "theme": {
-        "images": {
-          "headerURL": "image1.png",
+      theme: {
+        images: {
+          theme_frame: "image1.png",
         },
-        "colors": {
-          "accentcolor": ACCENT_COLOR,
-          "textcolor": TEXT_COLOR,
+        colors: {
+          frame: ACCENT_COLOR,
+          tab_background_text: TEXT_COLOR,
         },
       },
     },
@@ -30,22 +24,26 @@ add_task(async function test_multiple_windows() {
   await extension.startup();
 
   let docEl = window.document.documentElement;
-  let style = window.getComputedStyle(docEl);
 
   Assert.ok(docEl.hasAttribute("lwtheme"), "LWT attribute should be set");
-  Assert.equal(docEl.getAttribute("lwthemetextcolor"), "bright",
-               "LWT text color attribute should be set");
-  Assert.ok(style.backgroundImage.includes("image1.png"), "Expected background image");
+  Assert.equal(
+    docEl.getAttribute("lwthemetextcolor"),
+    "bright",
+    "LWT text color attribute should be set"
+  );
+  checkThemeHeaderImage(window, `moz-extension://${extension.uuid}/image1.png`);
 
   // Now we'll open a new window to see if the theme is also applied there.
   let window2 = await BrowserTestUtils.openNewBrowserWindow();
   docEl = window2.document.documentElement;
-  style = window2.getComputedStyle(docEl);
 
   Assert.ok(docEl.hasAttribute("lwtheme"), "LWT attribute should be set");
-  Assert.equal(docEl.getAttribute("lwthemetextcolor"), "bright",
-               "LWT text color attribute should be set");
-  Assert.ok(style.backgroundImage.includes("image1.png"), "Expected background image");
+  Assert.equal(
+    docEl.getAttribute("lwthemetextcolor"),
+    "bright",
+    "LWT text color attribute should be set"
+  );
+  checkThemeHeaderImage(window, `moz-extension://${extension.uuid}/image1.png`);
 
   await BrowserTestUtils.closeWindow(window2);
   await extension.unload();
