@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -36,104 +36,93 @@ namespace {
 // EventTarget and delegate their Run methods to a wrapped Runnable or lambda
 // function.
 
-class ChannelEventWrapper : public ChannelEvent
-{
-public:
-  ChannelEventWrapper(nsIEventTarget* aTarget)
-    : mTarget(aTarget)
-  {}
+class ChannelEventWrapper : public ChannelEvent {
+ public:
+  ChannelEventWrapper(nsIEventTarget* aTarget) : mTarget(aTarget) {}
 
-  already_AddRefed<nsIEventTarget> GetEventTarget() override
-  {
+  already_AddRefed<nsIEventTarget> GetEventTarget() override {
     return do_AddRef(mTarget);
   }
 
-protected:
+ protected:
   ~ChannelEventWrapper() override = default;
 
-private:
+ private:
   nsCOMPtr<nsIEventTarget> mTarget;
 };
 
-class ChannelEventFunction final : public ChannelEventWrapper
-{
-public:
+class ChannelEventFunction final : public ChannelEventWrapper {
+ public:
   ChannelEventFunction(nsIEventTarget* aTarget, std::function<void()>&& aFunc)
-    : ChannelEventWrapper(aTarget)
-    , mFunc(Move(aFunc))
-  {}
+      : ChannelEventWrapper(aTarget), mFunc(std::move(aFunc)) {}
 
-  void Run() override
-  {
-    mFunc();
-  }
+  void Run() override { mFunc(); }
 
-protected:
+ protected:
   ~ChannelEventFunction() override = default;
 
-private:
+ private:
   std::function<void()> mFunc;
 };
 
-class ChannelEventRunnable final : public ChannelEventWrapper
-{
-public:
-  ChannelEventRunnable(nsIEventTarget* aTarget, already_AddRefed<Runnable> aRunnable)
-    : ChannelEventWrapper(aTarget)
-    , mRunnable(aRunnable)
-  {}
+class ChannelEventRunnable final : public ChannelEventWrapper {
+ public:
+  ChannelEventRunnable(nsIEventTarget* aTarget,
+                       already_AddRefed<Runnable> aRunnable)
+      : ChannelEventWrapper(aTarget), mRunnable(aRunnable) {}
 
-  void Run() override
-  {
+  void Run() override {
     nsresult rv = mRunnable->Run();
     Unused << NS_WARN_IF(NS_FAILED(rv));
   }
 
-protected:
+ protected:
   ~ChannelEventRunnable() override = default;
 
-private:
+ private:
   RefPtr<Runnable> mRunnable;
 };
 
-} // anonymous namespace
+}  // anonymous namespace
 
 /*****************************************************************************
  * Initialization
  *****************************************************************************/
 
 StreamFilterParent::StreamFilterParent()
-  : mMainThread(GetCurrentThreadEventTarget())
-  , mIOThread(mMainThread)
-  , mQueue(new ChannelEventQueue(static_cast<nsIStreamListener*>(this)))
-  , mBufferMutex("StreamFilter buffer mutex")
-  , mReceivedStop(false)
-  , mSentStop(false)
-  , mContext(nullptr)
-  , mOffset(0)
-  , mState(State::Uninitialized)
-{}
+    : mMainThread(GetCurrentThreadEventTarget()),
+      mIOThread(mMainThread),
+      mQueue(new ChannelEventQueue(static_cast<nsIStreamListener*>(this))),
+      mBufferMutex("StreamFilter buffer mutex"),
+      mReceivedStop(false),
+      mSentStop(false),
+      mContext(nullptr),
+      mOffset(0),
+      mState(State::Uninitialized) {}
 
-StreamFilterParent::~StreamFilterParent()
-{
+StreamFilterParent::~StreamFilterParent() {
+  NS_ReleaseOnMainThreadSystemGroup("StreamFilterParent::mChannel",
+                                    mChannel.forget());
+  NS_ReleaseOnMainThreadSystemGroup("StreamFilterParent::mLoadGroup",
+                                    mLoadGroup.forget());
   NS_ReleaseOnMainThreadSystemGroup("StreamFilterParent::mOrigListener",
                                     mOrigListener.forget());
   NS_ReleaseOnMainThreadSystemGroup("StreamFilterParent::mContext",
                                     mContext.forget());
 }
 
-bool
-StreamFilterParent::Create(dom::ContentParent* aContentParent, uint64_t aChannelId, const nsAString& aAddonId,
-                           Endpoint<PStreamFilterChild>* aEndpoint)
-{
+bool StreamFilterParent::Create(dom::ContentParent* aContentParent,
+                                uint64_t aChannelId, const nsAString& aAddonId,
+                                Endpoint<PStreamFilterChild>* aEndpoint) {
   AssertIsMainThread();
 
   auto& webreq = WebRequestService::GetSingleton();
 
   RefPtr<nsAtom> addonId = NS_Atomize(aAddonId);
-  nsCOMPtr<nsITraceableChannel> channel = webreq.GetTraceableChannel(aChannelId, addonId, aContentParent);
+  nsCOMPtr<nsITraceableChannel> channel =
+      webreq.GetTraceableChannel(aChannelId, addonId, aContentParent);
 
-  RefPtr<nsHttpChannel> chan = do_QueryObject(channel);
+  RefPtr<mozilla::net::nsHttpChannel> chan = do_QueryObject(channel);
   NS_ENSURE_TRUE(chan, false);
 
   auto channelPid = chan->ProcessId();
@@ -141,31 +130,30 @@ StreamFilterParent::Create(dom::ContentParent* aContentParent, uint64_t aChannel
 
   Endpoint<PStreamFilterParent> parent;
   Endpoint<PStreamFilterChild> child;
-  nsresult rv = PStreamFilter::CreateEndpoints(channelPid,
-                                               aContentParent ? aContentParent->OtherPid()
-                                                              : base::GetCurrentProcId(),
-                                               &parent, &child);
+  nsresult rv = PStreamFilter::CreateEndpoints(
+      channelPid,
+      aContentParent ? aContentParent->OtherPid() : base::GetCurrentProcId(),
+      &parent, &child);
   NS_ENSURE_SUCCESS(rv, false);
 
-  if (!chan->AttachStreamFilter(Move(parent))) {
+  if (!chan->AttachStreamFilter(std::move(parent))) {
     return false;
   }
 
-  *aEndpoint = Move(child);
+  *aEndpoint = std::move(child);
   return true;
 }
 
-/* static */ void
-StreamFilterParent::Attach(nsIChannel* aChannel, ParentEndpoint&& aEndpoint)
-{
+/* static */
+void StreamFilterParent::Attach(nsIChannel* aChannel,
+                                ParentEndpoint&& aEndpoint) {
   auto self = MakeRefPtr<StreamFilterParent>();
 
   self->ActorThread()->Dispatch(
-    NewRunnableMethod<ParentEndpoint&&>("StreamFilterParent::Bind",
-                                        self,
-                                        &StreamFilterParent::Bind,
-                                        Move(aEndpoint)),
-    NS_DISPATCH_NORMAL);
+      NewRunnableMethod<ParentEndpoint&&>("StreamFilterParent::Bind", self,
+                                          &StreamFilterParent::Bind,
+                                          std::move(aEndpoint)),
+      NS_DISPATCH_NORMAL);
 
   self->Init(aChannel);
 
@@ -173,15 +161,11 @@ StreamFilterParent::Attach(nsIChannel* aChannel, ParentEndpoint&& aEndpoint)
   Unused << self.forget();
 }
 
-void
-StreamFilterParent::Bind(ParentEndpoint&& aEndpoint)
-{
+void StreamFilterParent::Bind(ParentEndpoint&& aEndpoint) {
   aEndpoint.Bind(this);
 }
 
-void
-StreamFilterParent::Init(nsIChannel* aChannel)
-{
+void StreamFilterParent::Init(nsIChannel* aChannel) {
   mChannel = aChannel;
 
   nsCOMPtr<nsITraceableChannel> traceable = do_QueryInterface(aChannel);
@@ -196,12 +180,11 @@ StreamFilterParent::Init(nsIChannel* aChannel)
  *****************************************************************************/
 
 NS_IMETHODIMP
-StreamFilterParent::CheckListenerChain()
-{
+StreamFilterParent::CheckListenerChain() {
   AssertIsMainThread();
 
   nsCOMPtr<nsIThreadRetargetableStreamListener> trsl =
-    do_QueryInterface(mOrigListener);
+      do_QueryInterface(mOrigListener);
   if (trsl) {
     return trsl->CheckListenerChain();
   }
@@ -212,33 +195,31 @@ StreamFilterParent::CheckListenerChain()
  * Error handling
  *****************************************************************************/
 
-void
-StreamFilterParent::Broken()
-{
+void StreamFilterParent::Broken() {
   AssertIsActorThread();
 
-  mState = State::Disconnecting;
-
-  RefPtr<StreamFilterParent> self(this);
-  RunOnIOThread(FUNC, [=] {
-    self->FlushBufferedData();
-
-    RunOnActorThread(FUNC, [=] {
-      if (self->IPCActive()) {
-        self->mDisconnected = true;
-        self->mState = State::Disconnected;
+  switch (mState) {
+    case State::Initialized:
+    case State::TransferringData:
+    case State::Suspended:
+      mState = State::Disconnecting;
+      if (mChannel) {
+        mChannel->Cancel(NS_ERROR_FAILURE);
       }
-    });
-  });
+
+      FinishDisconnect();
+      break;
+
+    default:
+      break;
+  }
 }
 
 /*****************************************************************************
  * State change requests
  *****************************************************************************/
 
-IPCResult
-StreamFilterParent::RecvClose()
-{
+IPCResult StreamFilterParent::RecvClose() {
   AssertIsActorThread();
 
   mState = State::Closed;
@@ -256,30 +237,21 @@ StreamFilterParent::RecvClose()
   return IPC_OK();
 }
 
-void
-StreamFilterParent::Destroy()
-{
+void StreamFilterParent::Destroy() {
   // Close the channel asynchronously so the actor is never destroyed before
   // this message is fully processed.
-  ActorThread()->Dispatch(
-    NewRunnableMethod("StreamFilterParent::Close",
-                      this,
-                      &StreamFilterParent::Close),
-    NS_DISPATCH_NORMAL);
+  ActorThread()->Dispatch(NewRunnableMethod("StreamFilterParent::Close", this,
+                                            &StreamFilterParent::Close),
+                          NS_DISPATCH_NORMAL);
 }
 
-IPCResult
-StreamFilterParent::RecvDestroy()
-{
+IPCResult StreamFilterParent::RecvDestroy() {
   AssertIsActorThread();
   Destroy();
   return IPC_OK();
 }
 
-
-IPCResult
-StreamFilterParent::RecvSuspend()
-{
+IPCResult StreamFilterParent::RecvSuspend() {
   AssertIsActorThread();
 
   if (mState == State::TransferringData) {
@@ -298,9 +270,7 @@ StreamFilterParent::RecvSuspend()
   return IPC_OK();
 }
 
-IPCResult
-StreamFilterParent::RecvResume()
-{
+IPCResult StreamFilterParent::RecvResume() {
   AssertIsActorThread();
 
   if (mState == State::Suspended) {
@@ -321,16 +291,12 @@ StreamFilterParent::RecvResume()
   }
   return IPC_OK();
 }
-IPCResult
-StreamFilterParent::RecvDisconnect()
-{
+IPCResult StreamFilterParent::RecvDisconnect() {
   AssertIsActorThread();
 
   if (mState == State::Suspended) {
     RefPtr<StreamFilterParent> self(this);
-    RunOnMainThread(FUNC, [=] {
-      self->mChannel->Resume();
-    });
+    RunOnMainThread(FUNC, [=] { self->mChannel->Resume(); });
   } else if (mState != State::TransferringData) {
     return IPC_OK();
   }
@@ -340,82 +306,154 @@ StreamFilterParent::RecvDisconnect()
   return IPC_OK();
 }
 
-IPCResult
-StreamFilterParent::RecvFlushedData()
-{
+IPCResult StreamFilterParent::RecvFlushedData() {
   AssertIsActorThread();
 
   MOZ_ASSERT(mState == State::Disconnecting);
 
   Destroy();
 
+  FinishDisconnect();
+  return IPC_OK();
+}
+
+void StreamFilterParent::FinishDisconnect() {
   RefPtr<StreamFilterParent> self(this);
   RunOnIOThread(FUNC, [=] {
     self->FlushBufferedData();
 
+    RunOnMainThread(FUNC, [=] {
+      if (self->mLoadGroup) {
+        Unused << self->mLoadGroup->RemoveRequest(self, nullptr, NS_OK);
+      }
+    });
+
     RunOnActorThread(FUNC, [=] {
-      self->mState = State::Disconnected;
-      self->mDisconnected = true;
+      if (self->mState != State::Closed) {
+        self->mState = State::Disconnected;
+        self->mDisconnected = true;
+      }
     });
   });
-  return IPC_OK();
 }
 
 /*****************************************************************************
  * Data output
  *****************************************************************************/
 
-IPCResult
-StreamFilterParent::RecvWrite(Data&& aData)
-{
+IPCResult StreamFilterParent::RecvWrite(Data&& aData) {
   AssertIsActorThread();
 
-
-  RunOnIOThread(
-    NewRunnableMethod<Data&&>("StreamFilterParent::WriteMove",
-                              this,
-                              &StreamFilterParent::WriteMove,
-                              Move(aData)));
+  RunOnIOThread(NewRunnableMethod<Data&&>("StreamFilterParent::WriteMove", this,
+                                          &StreamFilterParent::WriteMove,
+                                          std::move(aData)));
   return IPC_OK();
 }
 
-void
-StreamFilterParent::WriteMove(Data&& aData)
-{
+void StreamFilterParent::WriteMove(Data&& aData) {
   nsresult rv = Write(aData);
   Unused << NS_WARN_IF(NS_FAILED(rv));
 }
 
-nsresult
-StreamFilterParent::Write(Data& aData)
-{
+nsresult StreamFilterParent::Write(Data& aData) {
   AssertIsIOThread();
 
   nsCOMPtr<nsIInputStream> stream;
-  nsresult rv = NS_NewByteInputStream(getter_AddRefs(stream),
-                                      reinterpret_cast<char*>(aData.Elements()),
-                                      aData.Length());
+  nsresult rv = NS_NewByteInputStream(
+      getter_AddRefs(stream),
+      MakeSpan(reinterpret_cast<char*>(aData.Elements()), aData.Length()),
+      NS_ASSIGNMENT_DEPEND);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = mOrigListener->OnDataAvailable(mChannel, mContext, stream,
-                                      mOffset, aData.Length());
+  rv =
+      mOrigListener->OnDataAvailable(mChannel, stream, mOffset, aData.Length());
   NS_ENSURE_SUCCESS(rv, rv);
 
   mOffset += aData.Length();
   return NS_OK;
 }
 
+/*****************************************************************************
+ * nsIRequest
+ *****************************************************************************/
+
+NS_IMETHODIMP
+StreamFilterParent::GetName(nsACString& aName) {
+  MOZ_ASSERT(mChannel);
+  return mChannel->GetName(aName);
+}
+
+NS_IMETHODIMP
+StreamFilterParent::GetStatus(nsresult* aStatus) {
+  MOZ_ASSERT(mChannel);
+  return mChannel->GetStatus(aStatus);
+}
+
+NS_IMETHODIMP
+StreamFilterParent::IsPending(bool* aIsPending) {
+  switch (mState) {
+    case State::Initialized:
+    case State::TransferringData:
+    case State::Suspended:
+      *aIsPending = true;
+      break;
+    default:
+      *aIsPending = false;
+  }
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+StreamFilterParent::Cancel(nsresult aResult) {
+  MOZ_ASSERT(mChannel);
+  return mChannel->Cancel(aResult);
+}
+
+NS_IMETHODIMP
+StreamFilterParent::Suspend() {
+  MOZ_ASSERT(mChannel);
+  return mChannel->Suspend();
+}
+
+NS_IMETHODIMP
+StreamFilterParent::Resume() {
+  MOZ_ASSERT(mChannel);
+  return mChannel->Resume();
+}
+
+NS_IMETHODIMP
+StreamFilterParent::GetLoadGroup(nsILoadGroup** aLoadGroup) {
+  *aLoadGroup = mLoadGroup;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+StreamFilterParent::SetLoadGroup(nsILoadGroup* aLoadGroup) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+StreamFilterParent::GetLoadFlags(nsLoadFlags* aLoadFlags) {
+  MOZ_ASSERT(mChannel);
+  MOZ_TRY(mChannel->GetLoadFlags(aLoadFlags));
+  *aLoadFlags &= ~nsIChannel::LOAD_DOCUMENT_URI;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+StreamFilterParent::SetLoadFlags(nsLoadFlags aLoadFlags) {
+  MOZ_ASSERT(mChannel);
+  return mChannel->SetLoadFlags(aLoadFlags);
+}
+
 /*****************************************************************************
  * nsIStreamListener
  *****************************************************************************/
 
 NS_IMETHODIMP
-StreamFilterParent::OnStartRequest(nsIRequest* aRequest, nsISupports* aContext)
-{
+StreamFilterParent::OnStartRequest(nsIRequest* aRequest) {
   AssertIsMainThread();
 
-  mContext = aContext;
-
   if (aRequest != mChannel) {
     mDisconnected = true;
 
@@ -428,6 +466,30 @@ StreamFilterParent::OnStartRequest(nsIRequest* aRequest, nsISupports* aContext)
     });
   }
 
+  if (!mDisconnected) {
+    Unused << mChannel->GetLoadGroup(getter_AddRefs(mLoadGroup));
+    if (mLoadGroup) {
+      Unused << mLoadGroup->AddRequest(this, nullptr);
+    }
+  }
+
+  nsresult rv = mOrigListener->OnStartRequest(aRequest);
+
+  // Important: Do this only *after* running the next listener in the chain, so
+  // that we get the final delivery target after any retargeting that it may do.
+  if (nsCOMPtr<nsIThreadRetargetableRequest> req =
+          do_QueryInterface(aRequest)) {
+    nsCOMPtr<nsIEventTarget> thread;
+    Unused << req->GetDeliveryTarget(getter_AddRefs(thread));
+    if (thread) {
+      mIOThread = std::move(thread);
+    }
+  }
+
+  // Important: Do this *after* we have set the thread delivery target, or it is
+  // possible in rare circumstances for an extension to attempt to write data
+  // before the thread has been set up, even though there are several layers of
+  // asynchrony involved.
   if (!mDisconnected) {
     RefPtr<StreamFilterParent> self(this);
     RunOnActorThread(FUNC, [=] {
@@ -438,26 +500,11 @@ StreamFilterParent::OnStartRequest(nsIRequest* aRequest, nsISupports* aContext)
     });
   }
 
-  nsresult rv = mOrigListener->OnStartRequest(aRequest, aContext);
-
-  // Important: Do this only *after* running the next listener in the chain, so
-  // that we get the final delivery target after any retargeting that it may do.
-  if (nsCOMPtr<nsIThreadRetargetableRequest> req = do_QueryInterface(aRequest)) {
-    nsCOMPtr<nsIEventTarget> thread;
-    Unused << req->GetDeliveryTarget(getter_AddRefs(thread));
-    if (thread) {
-      mIOThread = Move(thread);
-    }
-  }
-
   return rv;
 }
 
 NS_IMETHODIMP
-StreamFilterParent::OnStopRequest(nsIRequest* aRequest,
-                                  nsISupports* aContext,
-                                  nsresult aStatusCode)
-{
+StreamFilterParent::OnStopRequest(nsIRequest* aRequest, nsresult aStatusCode) {
   AssertIsMainThread();
 
   mReceivedStop = true;
@@ -474,23 +521,25 @@ StreamFilterParent::OnStopRequest(nsIRequest* aRequest,
   return NS_OK;
 }
 
-nsresult
-StreamFilterParent::EmitStopRequest(nsresult aStatusCode)
-{
+nsresult StreamFilterParent::EmitStopRequest(nsresult aStatusCode) {
   AssertIsMainThread();
   MOZ_ASSERT(!mSentStop);
 
   mSentStop = true;
-  return mOrigListener->OnStopRequest(mChannel, mContext, aStatusCode);
+  nsresult rv = mOrigListener->OnStopRequest(mChannel, aStatusCode);
+
+  if (mLoadGroup && !mDisconnected) {
+    Unused << mLoadGroup->RemoveRequest(this, nullptr, aStatusCode);
+  }
+
+  return rv;
 }
 
 /*****************************************************************************
  * Incoming data handling
  *****************************************************************************/
 
-void
-StreamFilterParent::DoSendData(Data&& aData)
-{
+void StreamFilterParent::DoSendData(Data&& aData) {
   AssertIsActorThread();
 
   if (mState == State::TransferringData) {
@@ -500,11 +549,8 @@ StreamFilterParent::DoSendData(Data&& aData)
 
 NS_IMETHODIMP
 StreamFilterParent::OnDataAvailable(nsIRequest* aRequest,
-                                    nsISupports* aContext,
                                     nsIInputStream* aInputStream,
-                                    uint64_t aOffset,
-                                    uint32_t aCount)
-{
+                                    uint64_t aOffset, uint32_t aCount) {
   AssertIsIOThread();
 
   if (mDisconnected) {
@@ -520,7 +566,7 @@ StreamFilterParent::OnDataAvailable(nsIRequest* aRequest,
     }
 
     mOffset += aCount;
-    return mOrigListener->OnDataAvailable(aRequest, aContext, aInputStream,
+    return mOrigListener->OnDataAvailable(aRequest, aInputStream,
                                           mOffset - aCount, aCount);
   }
 
@@ -535,23 +581,20 @@ StreamFilterParent::OnDataAvailable(nsIRequest* aRequest,
 
   if (mState == State::Disconnecting) {
     MutexAutoLock al(mBufferMutex);
-    BufferData(Move(data));
+    BufferData(std::move(data));
   } else if (mState == State::Closed) {
     return NS_ERROR_FAILURE;
   } else {
     ActorThread()->Dispatch(
-      NewRunnableMethod<Data&&>("StreamFilterParent::DoSendData",
-                                this,
-                                &StreamFilterParent::DoSendData,
-                                Move(data)),
-      NS_DISPATCH_NORMAL);
+        NewRunnableMethod<Data&&>("StreamFilterParent::DoSendData", this,
+                                  &StreamFilterParent::DoSendData,
+                                  std::move(data)),
+        NS_DISPATCH_NORMAL);
   }
   return NS_OK;
 }
 
-nsresult
-StreamFilterParent::FlushBufferedData()
-{
+nsresult StreamFilterParent::FlushBufferedData() {
   AssertIsIOThread();
 
   // When offloading data to a thread pool, OnDataAvailable isn't guaranteed
@@ -583,72 +626,44 @@ StreamFilterParent::FlushBufferedData()
  * Thread helpers
  *****************************************************************************/
 
-nsIEventTarget*
-StreamFilterParent::ActorThread()
-{
-  return gSocketTransportService;
+nsIEventTarget* StreamFilterParent::ActorThread() {
+  return net::gSocketTransportService;
 }
 
-bool
-StreamFilterParent::IsActorThread()
-{
+bool StreamFilterParent::IsActorThread() {
   return ActorThread()->IsOnCurrentThread();
 }
 
-void
-StreamFilterParent::AssertIsActorThread()
-{
-  MOZ_ASSERT(IsActorThread());
+void StreamFilterParent::AssertIsActorThread() { MOZ_ASSERT(IsActorThread()); }
+
+nsIEventTarget* StreamFilterParent::IOThread() { return mIOThread; }
+
+bool StreamFilterParent::IsIOThread() { return mIOThread->IsOnCurrentThread(); }
+
+void StreamFilterParent::AssertIsIOThread() { MOZ_ASSERT(IsIOThread()); }
+
+template <typename Function>
+void StreamFilterParent::RunOnMainThread(const char* aName, Function&& aFunc) {
+  mQueue->RunOrEnqueue(new ChannelEventFunction(mMainThread, std::move(aFunc)));
 }
 
-nsIEventTarget*
-StreamFilterParent::IOThread()
-{
-  return mIOThread;
+void StreamFilterParent::RunOnMainThread(already_AddRefed<Runnable> aRunnable) {
+  mQueue->RunOrEnqueue(
+      new ChannelEventRunnable(mMainThread, std::move(aRunnable)));
 }
 
-bool
-StreamFilterParent::IsIOThread()
-{
-  return mIOThread->IsOnCurrentThread();
+template <typename Function>
+void StreamFilterParent::RunOnIOThread(const char* aName, Function&& aFunc) {
+  mQueue->RunOrEnqueue(new ChannelEventFunction(mIOThread, std::move(aFunc)));
 }
 
-void
-StreamFilterParent::AssertIsIOThread()
-{
-  MOZ_ASSERT(IsIOThread());
+void StreamFilterParent::RunOnIOThread(already_AddRefed<Runnable> aRunnable) {
+  mQueue->RunOrEnqueue(
+      new ChannelEventRunnable(mIOThread, std::move(aRunnable)));
 }
 
-template<typename Function>
-void
-StreamFilterParent::RunOnMainThread(const char* aName, Function&& aFunc)
-{
-  mQueue->RunOrEnqueue(new ChannelEventFunction(mMainThread, Move(aFunc)));
-}
-
-void
-StreamFilterParent::RunOnMainThread(already_AddRefed<Runnable> aRunnable)
-{
-  mQueue->RunOrEnqueue(new ChannelEventRunnable(mMainThread, Move(aRunnable)));
-}
-
-template<typename Function>
-void
-StreamFilterParent::RunOnIOThread(const char* aName, Function&& aFunc)
-{
-  mQueue->RunOrEnqueue(new ChannelEventFunction(mIOThread, Move(aFunc)));
-}
-
-void
-StreamFilterParent::RunOnIOThread(already_AddRefed<Runnable> aRunnable)
-{
-  mQueue->RunOrEnqueue(new ChannelEventRunnable(mIOThread, Move(aRunnable)));
-}
-
-template<typename Function>
-void
-StreamFilterParent::RunOnActorThread(const char* aName, Function&& aFunc)
-{
+template <typename Function>
+void StreamFilterParent::RunOnActorThread(const char* aName, Function&& aFunc) {
   // We don't use mQueue for dispatch to the actor thread.
   //
   // The main thread and IO thread are used for dispatching events to the
@@ -667,9 +682,8 @@ StreamFilterParent::RunOnActorThread(const char* aName, Function&& aFunc)
   if (IsActorThread()) {
     aFunc();
   } else {
-    ActorThread()->Dispatch(
-      Move(NS_NewRunnableFunction(aName, aFunc)),
-      NS_DISPATCH_NORMAL);
+    ActorThread()->Dispatch(std::move(NS_NewRunnableFunction(aName, aFunc)),
+                            NS_DISPATCH_NORMAL);
   }
 }
 
@@ -677,9 +691,7 @@ StreamFilterParent::RunOnActorThread(const char* aName, Function&& aFunc)
  * Glue
  *****************************************************************************/
 
-void
-StreamFilterParent::ActorDestroy(ActorDestroyReason aWhy)
-{
+void StreamFilterParent::ActorDestroy(ActorDestroyReason aWhy) {
   AssertIsActorThread();
 
   if (mState != State::Disconnected && mState != State::Closed) {
@@ -687,14 +699,20 @@ StreamFilterParent::ActorDestroy(ActorDestroyReason aWhy)
   }
 }
 
-void
-StreamFilterParent::DeallocPStreamFilterParent()
-{
+void StreamFilterParent::DeallocPStreamFilterParent() {
   RefPtr<StreamFilterParent> self = dont_AddRef(this);
 }
 
-NS_IMPL_ISUPPORTS(StreamFilterParent, nsIStreamListener, nsIRequestObserver, nsIThreadRetargetableStreamListener)
+NS_INTERFACE_MAP_BEGIN(StreamFilterParent)
+  NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
+  NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
+  NS_INTERFACE_MAP_ENTRY(nsIRequest)
+  NS_INTERFACE_MAP_ENTRY(nsIThreadRetargetableStreamListener)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIStreamListener)
+NS_INTERFACE_MAP_END
 
-} // namespace extensions
-} // namespace mozilla
+NS_IMPL_ADDREF(StreamFilterParent)
+NS_IMPL_RELEASE(StreamFilterParent)
 
+}  // namespace extensions
+}  // namespace mozilla
