@@ -26,16 +26,13 @@
 #include "nsIClassInfoImpl.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/LoadInfo.h"
+#include "mozilla/NullPrincipal.h"
 #include "mozilla/Preferences.h"
 #include "nsILoadInfo.h"
 #include "nsIContentPolicy.h"
 #include "nsContentUtils.h"
-#include "NullPrincipal.h"
 #include "imgICache.h"
 
-#define MAX_FAILED_FAVICONS 256
-#define FAVICON_CACHE_REDUCE_COUNT 64
-
 #define UNASSOCIATED_FAVICONS_LENGTH 32
 
 // When replaceFaviconData is called, we store the icons in an in-memory cache
@@ -46,13 +43,14 @@
 using namespace mozilla;
 using namespace mozilla::places;
 
+const uint16_t gFaviconSizes[7] = {192, 144, 96, 64, 48, 32, 16};
+
 /**
  * Used to notify a topic to system observers on async execute completion.
  * Will throw on error.
  */
-class ExpireFaviconsStatementCallbackNotifier : public AsyncStatementCallback
-{
-public:
+class ExpireFaviconsStatementCallbackNotifier : public AsyncStatementCallback {
+ public:
   ExpireFaviconsStatementCallbackNotifier();
   NS_IMETHOD HandleCompletion(uint16_t aReason) override;
 };
@@ -65,9 +63,8 @@ namespace {
  * All calculation is done considering square sizes and the largest side.
  * In case of multiple frames of the same size, only the first one is retained.
  */
-nsresult
-GetFramesInfoForContainer(imgIContainer* aContainer,
-                           nsTArray<FrameData>& aFramesInfo) {
+nsresult GetFramesInfoForContainer(imgIContainer* aContainer,
+                                   nsTArray<FrameData>& aFramesInfo) {
   // Don't extract frames from animated images.
   bool animated;
   nsresult rv = aContainer->GetAnimated(&animated);
@@ -82,14 +79,14 @@ GetFramesInfoForContainer(imgIContainer* aContainer,
           continue;
         }
         // Check if it's one of the sizes we care about.
-        auto end = std::end(sFaviconSizes);
-        uint16_t* matchingSize = std::find(std::begin(sFaviconSizes), end,
-                                          nativeSize.width);
+        auto end = std::end(gFaviconSizes);
+        const uint16_t* matchingSize =
+            std::find(std::begin(gFaviconSizes), end, nativeSize.width);
         if (matchingSize != end) {
-          // We must avoid duped sizes, an image could contain multiple frames of
-          // the same size, but we can only store one. We could use an hashtable,
-          // but considered the average low number of frames, we'll just do a
-          // linear search.
+          // We must avoid duped sizes, an image could contain multiple frames
+          // of the same size, but we can only store one. We could use an
+          // hashtable, but considered the average low number of frames, we'll
+          // just do a linear search.
           bool dupe = false;
           for (const auto& frameInfo : aFramesInfo) {
             if (frameInfo.width == *matchingSize) {
@@ -119,51 +116,38 @@ GetFramesInfoForContainer(imgIContainer* aContainer,
   return NS_OK;
 }
 
-} // namespace
+}  // namespace
 
 PLACES_FACTORY_SINGLETON_IMPLEMENTATION(nsFaviconService, gFaviconService)
 
 NS_IMPL_CLASSINFO(nsFaviconService, nullptr, 0, NS_FAVICONSERVICE_CID)
-NS_IMPL_ISUPPORTS_CI(
-  nsFaviconService
-, nsIFaviconService
-, mozIAsyncFavicons
-, nsITimerCallback
-, nsINamed
-)
+NS_IMPL_ISUPPORTS_CI(nsFaviconService, nsIFaviconService, nsITimerCallback,
+                     nsINamed)
 
 nsFaviconService::nsFaviconService()
-  : mFailedFaviconSerial(0)
-  , mFailedFavicons(MAX_FAILED_FAVICONS / 2)
-  , mUnassociatedIcons(UNASSOCIATED_FAVICONS_LENGTH)
-  , mDefaultIconURIPreferredSize(UINT16_MAX)
-{
+    : mUnassociatedIcons(UNASSOCIATED_FAVICONS_LENGTH),
+      mDefaultIconURIPreferredSize(UINT16_MAX) {
   NS_ASSERTION(!gFaviconService,
                "Attempting to create two instances of the service!");
   gFaviconService = this;
 }
 
-
-nsFaviconService::~nsFaviconService()
-{
+nsFaviconService::~nsFaviconService() {
   NS_ASSERTION(gFaviconService == this,
                "Deleting a non-singleton instance of the service");
-  if (gFaviconService == this)
-    gFaviconService = nullptr;
+  if (gFaviconService == this) gFaviconService = nullptr;
 }
 
 Atomic<int64_t> nsFaviconService::sLastInsertedIconId(0);
 
-void // static
+void  // static
 nsFaviconService::StoreLastInsertedId(const nsACString& aTable,
                                       const int64_t aLastInsertedId) {
   MOZ_ASSERT(aTable.EqualsLiteral("moz_icons"));
   sLastInsertedIconId = aLastInsertedId;
 }
 
-nsresult
-nsFaviconService::Init()
-{
+nsresult nsFaviconService::Init() {
   mDB = Database::GetDatabase();
   NS_ENSURE_STATE(mDB);
 
@@ -172,7 +156,7 @@ nsFaviconService::Init()
 
   // Check if there are still icon payloads to be converted.
   bool shouldConvertPayloads =
-    Preferences::GetBool(PREF_CONVERT_PAYLOADS, false);
+      Preferences::GetBool(PREF_CONVERT_PAYLOADS, false);
   if (shouldConvertPayloads) {
     ConvertUnsupportedPayloads(mDB->MainConn());
   }
@@ -181,45 +165,37 @@ nsFaviconService::Init()
 }
 
 NS_IMETHODIMP
-nsFaviconService::ExpireAllFavicons()
-{
+nsFaviconService::ExpireAllFavicons() {
   NS_ENSURE_STATE(mDB);
 
-  nsCOMPtr<mozIStorageAsyncStatement> removePagesStmt = mDB->GetAsyncStatement(
-    "DELETE FROM moz_pages_w_icons"
-  );
+  nsCOMPtr<mozIStorageAsyncStatement> removePagesStmt =
+      mDB->GetAsyncStatement("DELETE FROM moz_pages_w_icons");
   NS_ENSURE_STATE(removePagesStmt);
-  nsCOMPtr<mozIStorageAsyncStatement> removeIconsStmt = mDB->GetAsyncStatement(
-    "DELETE FROM moz_icons"
-  );
+  nsCOMPtr<mozIStorageAsyncStatement> removeIconsStmt =
+      mDB->GetAsyncStatement("DELETE FROM moz_icons");
   NS_ENSURE_STATE(removeIconsStmt);
-  nsCOMPtr<mozIStorageAsyncStatement> unlinkIconsStmt = mDB->GetAsyncStatement(
-    "DELETE FROM moz_icons_to_pages"
-  );
+  nsCOMPtr<mozIStorageAsyncStatement> unlinkIconsStmt =
+      mDB->GetAsyncStatement("DELETE FROM moz_icons_to_pages");
   NS_ENSURE_STATE(unlinkIconsStmt);
 
   mozIStorageBaseStatement* stmts[] = {
-    removePagesStmt.get()
-  , removeIconsStmt.get()
-  , unlinkIconsStmt.get()
-  };
+      removePagesStmt.get(), removeIconsStmt.get(), unlinkIconsStmt.get()};
   nsCOMPtr<mozIStorageConnection> conn = mDB->MainConn();
   if (!conn) {
     return NS_ERROR_UNEXPECTED;
   }
   nsCOMPtr<mozIStoragePendingStatement> ps;
   RefPtr<ExpireFaviconsStatementCallbackNotifier> callback =
-    new ExpireFaviconsStatementCallbackNotifier();
-  return conn->ExecuteAsync(stmts, ArrayLength(stmts),
-                                        callback, getter_AddRefs(ps));
+      new ExpireFaviconsStatementCallbackNotifier();
+  return conn->ExecuteAsync(stmts, ArrayLength(stmts), callback,
+                            getter_AddRefs(ps));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 //// nsITimerCallback
 
 NS_IMETHODIMP
-nsFaviconService::Notify(nsITimer* timer)
-{
+nsFaviconService::Notify(nsITimer* timer) {
   if (timer != mExpireUnassociatedIconsTimer.get()) {
     return NS_ERROR_INVALID_ARG;
   }
@@ -235,7 +211,7 @@ nsFaviconService::Notify(nsITimer* timer)
   // Re-init the expiry timer if the cache isn't empty.
   if (mUnassociatedIcons.Count() > 0) {
     mExpireUnassociatedIconsTimer->InitWithCallback(
-      this, UNASSOCIATED_ICON_EXPIRY_INTERVAL, nsITimer::TYPE_ONE_SHOT);
+        this, UNASSOCIATED_ICON_EXPIRY_INTERVAL, nsITimer::TYPE_ONE_SHOT);
   }
 
   return NS_OK;
@@ -245,8 +221,7 @@ nsFaviconService::Notify(nsITimer* timer)
 //// nsINamed
 
 NS_IMETHODIMP
-nsFaviconService::GetName(nsACString& aName)
-{
+nsFaviconService::GetName(nsACString& aName) {
   aName.AssignLiteral("nsFaviconService");
   return NS_OK;
 }
@@ -255,8 +230,7 @@ nsFaviconService::GetName(nsACString& aName)
 //// nsIFaviconService
 
 NS_IMETHODIMP
-nsFaviconService::GetDefaultFavicon(nsIURI** _retval)
-{
+nsFaviconService::GetDefaultFavicon(nsIURI** _retval) {
   NS_ENSURE_ARG_POINTER(_retval);
 
   // not found, use default
@@ -265,21 +239,21 @@ nsFaviconService::GetDefaultFavicon(nsIURI** _retval)
                             NS_LITERAL_CSTRING(FAVICON_DEFAULT_URL));
     NS_ENSURE_SUCCESS(rv, rv);
   }
-  return mDefaultIcon->Clone(_retval);
+
+  nsCOMPtr<nsIURI> uri = mDefaultIcon;
+  uri.forget(_retval);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFaviconService::GetDefaultFaviconMimeType(nsACString& _retval)
-{
+nsFaviconService::GetDefaultFaviconMimeType(nsACString& _retval) {
   _retval = NS_LITERAL_CSTRING(FAVICON_DEFAULT_MIMETYPE);
   return NS_OK;
 }
 
-void
-nsFaviconService::SendFaviconNotifications(nsIURI* aPageURI,
-                                           nsIURI* aFaviconURI,
-                                           const nsACString& aGUID)
-{
+void nsFaviconService::SendFaviconNotifications(nsIURI* aPageURI,
+                                                nsIURI* aFaviconURI,
+                                                const nsACString& aGUID) {
   nsAutoCString faviconSpec;
   nsNavHistory* history = nsNavHistory::GetHistoryService();
   if (history && NS_SUCCEEDED(aFaviconURI->GetSpec(faviconSpec))) {
@@ -295,7 +269,8 @@ nsFaviconService::SendFaviconNotifications(nsIURI* aPageURI,
       MOZ_ASSERT(NS_SUCCEEDED(rv));
       if (NS_SUCCEEDED(rv)) {
         nsCOMPtr<imgICache> imgCache;
-        rv = GetImgTools()->GetImgCacheForDocument(nullptr, getter_AddRefs(imgCache));
+        rv = GetImgTools()->GetImgCacheForDocument(nullptr,
+                                                   getter_AddRefs(imgCache));
         MOZ_ASSERT(NS_SUCCEEDED(rv));
         if (NS_SUCCEEDED(rv)) {
           Unused << imgCache->RemoveEntry(pageIconURI, nullptr);
@@ -303,64 +278,47 @@ nsFaviconService::SendFaviconNotifications(nsIURI* aPageURI,
       }
     }
 
-    history->SendPageChangedNotification(aPageURI,
-                                         nsINavHistoryObserver::ATTRIBUTE_FAVICON,
-                                         NS_ConvertUTF8toUTF16(faviconSpec),
-                                         aGUID);
+    history->SendPageChangedNotification(
+        aPageURI, nsINavHistoryObserver::ATTRIBUTE_FAVICON,
+        NS_ConvertUTF8toUTF16(faviconSpec), aGUID);
   }
 }
 
 NS_IMETHODIMP
-nsFaviconService::SetAndFetchFaviconForPage(nsIURI* aPageURI,
-                                            nsIURI* aFaviconURI,
-                                            bool aForceReload,
-                                            uint32_t aFaviconLoadType,
-                                            nsIFaviconDataCallback* aCallback,
-                                            nsIPrincipal* aLoadingPrincipal,
-                                            uint64_t aRequestContextID,
-                                            mozIPlacesPendingOperation **_canceler)
-{
+nsFaviconService::SetAndFetchFaviconForPage(
+    nsIURI* aPageURI, nsIURI* aFaviconURI, bool aForceReload,
+    uint32_t aFaviconLoadType, nsIFaviconDataCallback* aCallback,
+    nsIPrincipal* aLoadingPrincipal, uint64_t aRequestContextID,
+    mozIPlacesPendingOperation** _canceler) {
   MOZ_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG(aPageURI);
   NS_ENSURE_ARG(aFaviconURI);
   NS_ENSURE_ARG_POINTER(_canceler);
 
-  // If a favicon is in the failed cache, only load it during a forced reload.
-  bool previouslyFailed;
-  nsresult rv = IsFailedFavicon(aFaviconURI, &previouslyFailed);
-  NS_ENSURE_SUCCESS(rv, rv);
-  if (previouslyFailed) {
-    if (aForceReload) {
-      RemoveFailedFavicon(aFaviconURI);
-    }
-    else {
-      return NS_OK;
-    }
-  }
-
   nsCOMPtr<nsIPrincipal> loadingPrincipal = aLoadingPrincipal;
-  MOZ_ASSERT(loadingPrincipal, "please provide aLoadingPrincipal for this favicon");
+  MOZ_ASSERT(loadingPrincipal,
+             "please provide aLoadingPrincipal for this favicon");
   if (!loadingPrincipal) {
     // Let's default to the nullPrincipal if no loadingPrincipal is provided.
     const char16_t* params[] = {
-      u"nsFaviconService::setAndFetchFaviconForPage()",
-      u"nsFaviconService::setAndFetchFaviconForPage(..., [optional aLoadingPrincipal])"
-    };
-    nsContentUtils::ReportToConsole(nsIScriptError::warningFlag,
-                                    NS_LITERAL_CSTRING("Security by Default"),
-                                    nullptr, // aDocument
-                                    nsContentUtils::eNECKO_PROPERTIES,
-                                    "APIDeprecationWarning",
-                                    params, ArrayLength(params));
-    loadingPrincipal = NullPrincipal::Create();
+        u"nsFaviconService::setAndFetchFaviconForPage()",
+        u"nsFaviconService::setAndFetchFaviconForPage(..., [optional "
+        u"aLoadingPrincipal])"};
+    nsContentUtils::ReportToConsole(
+        nsIScriptError::warningFlag, NS_LITERAL_CSTRING("Security by Default"),
+        nullptr,  // aDocument
+        nsContentUtils::eNECKO_PROPERTIES, "APIDeprecationWarning", params,
+        ArrayLength(params));
+    loadingPrincipal = NullPrincipal::CreateWithoutOriginAttributes();
   }
   NS_ENSURE_TRUE(loadingPrincipal, NS_ERROR_FAILURE);
 
-  bool loadPrivate = aFaviconLoadType == nsIFaviconService::FAVICON_LOAD_PRIVATE;
+  bool loadPrivate =
+      aFaviconLoadType == nsIFaviconService::FAVICON_LOAD_PRIVATE;
 
   // Build page data.
   PageData page;
-  rv = aPageURI->GetSpec(page.spec);
+  nsresult rv = aPageURI->GetSpec(page.spec);
   NS_ENSURE_SUCCESS(rv, rv);
   // URIs can arguably lack a host.
   Unused << aPageURI->GetHost(page.host);
@@ -390,11 +348,16 @@ nsFaviconService::SetAndFetchFaviconForPage(nsIURI* aPageURI,
     if (StringBeginsWith(icon.host, NS_LITERAL_CSTRING("www."))) {
       icon.host.Cut(0, 4);
     }
-    nsAutoCString path;
-    rv = aFaviconURI->GetPathQueryRef(path);
-    if (NS_SUCCEEDED(rv) && path.EqualsLiteral("/favicon.ico")) {
-      icon.rootIcon = 1;
-    }
+  }
+
+  // A root icon is when the icon and page have the same host and the path
+  // is just /favicon.ico. These icons are considered valid for the whole
+  // origin and expired with the origin through a trigger.
+  nsAutoCString path;
+  if (NS_SUCCEEDED(aFaviconURI->GetPathQueryRef(path)) &&
+      !icon.host.IsEmpty() && icon.host.Equals(page.host) &&
+      path.EqualsLiteral("/favicon.ico")) {
+    icon.rootIcon = 1;
   }
 
   // If the page url points to an image, the icon's url will be the same.
@@ -406,10 +369,8 @@ nsFaviconService::SetAndFetchFaviconForPage(nsIURI* aPageURI,
     return NS_OK;
   }
 
-  RefPtr<AsyncFetchAndSetIconForPage> event =
-    new AsyncFetchAndSetIconForPage(icon, page, loadPrivate,
-                                    aCallback, aLoadingPrincipal,
-                                    aRequestContextID);
+  RefPtr<AsyncFetchAndSetIconForPage> event = new AsyncFetchAndSetIconForPage(
+      icon, page, loadPrivate, aCallback, aLoadingPrincipal, aRequestContextID);
 
   // Get the target thread and start the work.
   // DB will be updated and observers notified when data has finished loading.
@@ -425,18 +386,16 @@ nsFaviconService::SetAndFetchFaviconForPage(nsIURI* aPageURI,
 
 NS_IMETHODIMP
 nsFaviconService::ReplaceFaviconData(nsIURI* aFaviconURI,
-                                    const uint8_t* aData,
-                                    uint32_t aDataLen,
-                                    const nsACString& aMimeType,
-                                    PRTime aExpiration)
-{
+                                     const nsTArray<uint8_t>& aData,
+                                     const nsACString& aMimeType,
+                                     PRTime aExpiration) {
   MOZ_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG(aFaviconURI);
-  NS_ENSURE_ARG(aData);
-  NS_ENSURE_ARG(aDataLen > 0);
+  NS_ENSURE_ARG(aData.Length() > 0);
   NS_ENSURE_ARG(aMimeType.Length() > 0);
-  NS_ENSURE_ARG(imgLoader::SupportImageWithMimeType(PromiseFlatCString(aMimeType).get(),
-                                                     AcceptedMimeTypes::IMAGES_AND_DOCUMENTS));
+  NS_ENSURE_ARG(imgLoader::SupportImageWithMimeType(
+      PromiseFlatCString(aMimeType).get(),
+      AcceptedMimeTypes::IMAGES_AND_DOCUMENTS));
 
   if (aExpiration == 0) {
     aExpiration = PR_Now() + MAX_FAVICON_EXPIRATION;
@@ -449,13 +408,14 @@ nsFaviconService::ReplaceFaviconData(nsIURI* aFaviconURI,
 
   iconKey->created = PR_Now();
 
-  // If the cache contains unassociated icons, an expiry timer should already exist, otherwise
-  // there may be a timer left hanging around, so make sure we fire a new one.
+  // If the cache contains unassociated icons, an expiry timer should already
+  // exist, otherwise there may be a timer left hanging around, so make sure we
+  // fire a new one.
   int32_t unassociatedCount = mUnassociatedIcons.Count();
   if (unassociatedCount == 1) {
     mExpireUnassociatedIconsTimer->Cancel();
     mExpireUnassociatedIconsTimer->InitWithCallback(
-      this, UNASSOCIATED_ICON_EXPIRY_INTERVAL, nsITimer::TYPE_ONE_SHOT);
+        this, UNASSOCIATED_ICON_EXPIRY_INTERVAL, nsITimer::TYPE_ONE_SHOT);
   }
 
   IconData* iconData = &(iconKey->iconData);
@@ -464,20 +424,21 @@ nsFaviconService::ReplaceFaviconData(nsIURI* aFaviconURI,
   iconData->fetchMode = FETCH_NEVER;
   nsresult rv = aFaviconURI->GetSpec(iconData->spec);
   NS_ENSURE_SUCCESS(rv, rv);
-  nsAutoCString path;
-  rv = aFaviconURI->GetPathQueryRef(path);
-  if (NS_SUCCEEDED(rv) && path.EqualsLiteral("/favicon.ico")) {
-    iconData->rootIcon = 1;
-  }
   // URIs can arguably lack a host.
   Unused << aFaviconURI->GetHost(iconData->host);
   if (StringBeginsWith(iconData->host, NS_LITERAL_CSTRING("www."))) {
     iconData->host.Cut(0, 4);
   }
 
+  // Note we can't set rootIcon here, because don't know the page it will be
+  // associated with. We'll do that later in SetAndFetchFaviconForPage if the
+  // icon doesn't exist; otherwise, if AsyncReplaceFaviconData updates an
+  // existing icon, it will take care of not overwriting an existing
+  // root = 1 value.
+
   IconPayload payload;
   payload.mimeType = aMimeType;
-  payload.data.Assign(TO_CHARBUFFER(aData), aDataLen);
+  payload.data.Assign(TO_CHARBUFFER(aData.Elements()), aData.Length());
   if (payload.mimeType.EqualsLiteral(SVG_MIME_TYPE)) {
     payload.width = UINT16_MAX;
   }
@@ -499,7 +460,8 @@ nsFaviconService::ReplaceFaviconData(nsIURI* aFaviconURI,
   // If the database contains an icon at the given url, we will update the
   // database immediately so that the associated pages are kept in sync.
   // Otherwise, do nothing and let the icon be picked up from the memory hash.
-  RefPtr<AsyncReplaceFaviconData> event = new AsyncReplaceFaviconData(*iconData);
+  RefPtr<AsyncReplaceFaviconData> event =
+      new AsyncReplaceFaviconData(*iconData);
   RefPtr<Database> DB = Database::GetDatabase();
   NS_ENSURE_STATE(DB);
   DB->DispatchToAsyncThread(event);
@@ -508,11 +470,9 @@ nsFaviconService::ReplaceFaviconData(nsIURI* aFaviconURI,
 }
 
 NS_IMETHODIMP
-nsFaviconService::ReplaceFaviconDataFromDataURL(nsIURI* aFaviconURI,
-                                               const nsAString& aDataURL,
-                                               PRTime aExpiration,
-                                               nsIPrincipal* aLoadingPrincipal)
-{
+nsFaviconService::ReplaceFaviconDataFromDataURL(
+    nsIURI* aFaviconURI, const nsAString& aDataURL, PRTime aExpiration,
+    nsIPrincipal* aLoadingPrincipal) {
   NS_ENSURE_ARG(aFaviconURI);
   NS_ENSURE_TRUE(aDataURL.Length() > 0, NS_ERROR_INVALID_ARG);
   if (aExpiration == 0) {
@@ -531,40 +491,39 @@ nsFaviconService::ReplaceFaviconDataFromDataURL(nsIURI* aFaviconURI,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIPrincipal> loadingPrincipal = aLoadingPrincipal;
-  MOZ_ASSERT(loadingPrincipal, "please provide aLoadingPrincipal for this favicon");
+  MOZ_ASSERT(loadingPrincipal,
+             "please provide aLoadingPrincipal for this favicon");
   if (!loadingPrincipal) {
     // Let's default to the nullPrincipal if no loadingPrincipal is provided.
     const char16_t* params[] = {
-      u"nsFaviconService::ReplaceFaviconDataFromDataURL()",
-      u"nsFaviconService::ReplaceFaviconDataFromDataURL(..., [optional aLoadingPrincipal])"
-    };
-    nsContentUtils::ReportToConsole(nsIScriptError::warningFlag,
-                                    NS_LITERAL_CSTRING("Security by Default"),
-                                    nullptr, // aDocument
-                                    nsContentUtils::eNECKO_PROPERTIES,
-                                    "APIDeprecationWarning",
-                                    params, ArrayLength(params));
+        u"nsFaviconService::ReplaceFaviconDataFromDataURL()",
+        u"nsFaviconService::ReplaceFaviconDataFromDataURL(..., [optional "
+        u"aLoadingPrincipal])"};
+    nsContentUtils::ReportToConsole(
+        nsIScriptError::warningFlag, NS_LITERAL_CSTRING("Security by Default"),
+        nullptr,  // aDocument
+        nsContentUtils::eNECKO_PROPERTIES, "APIDeprecationWarning", params,
+        ArrayLength(params));
 
-    loadingPrincipal = NullPrincipal::Create();
+    loadingPrincipal = NullPrincipal::CreateWithoutOriginAttributes();
   }
   NS_ENSURE_TRUE(loadingPrincipal, NS_ERROR_FAILURE);
 
-  nsCOMPtr<nsILoadInfo> loadInfo =
-    new mozilla::LoadInfo(loadingPrincipal,
-                          nullptr, // aTriggeringPrincipal
-                          nullptr, // aLoadingNode
-                          nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_INHERITS |
-                          nsILoadInfo::SEC_ALLOW_CHROME |
-                          nsILoadInfo::SEC_DISALLOW_SCRIPT,
-                          nsIContentPolicy::TYPE_INTERNAL_IMAGE_FAVICON);
+  nsCOMPtr<nsILoadInfo> loadInfo = new mozilla::net::LoadInfo(
+      loadingPrincipal,
+      nullptr,  // aTriggeringPrincipal
+      nullptr,  // aLoadingNode
+      nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_INHERITS |
+          nsILoadInfo::SEC_ALLOW_CHROME | nsILoadInfo::SEC_DISALLOW_SCRIPT,
+      nsIContentPolicy::TYPE_INTERNAL_IMAGE_FAVICON);
 
   nsCOMPtr<nsIChannel> channel;
-  rv = protocolHandler->NewChannel2(dataURI, loadInfo, getter_AddRefs(channel));
+  rv = protocolHandler->NewChannel(dataURI, loadInfo, getter_AddRefs(channel));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Blocking stream is OK for data URIs.
   nsCOMPtr<nsIInputStream> stream;
-  rv = channel->Open2(getter_AddRefs(stream));
+  rv = channel->Open(getter_AddRefs(stream));
   NS_ENSURE_SUCCESS(rv, rv);
 
   uint64_t available64;
@@ -575,37 +534,31 @@ nsFaviconService::ReplaceFaviconDataFromDataURL(nsIURI* aFaviconURI,
   uint32_t available = (uint32_t)available64;
 
   // Read all the decoded data.
-  uint8_t* buffer = static_cast<uint8_t*>
-                               (moz_xmalloc(sizeof(uint8_t) * available));
-  if (!buffer)
-    return NS_ERROR_OUT_OF_MEMORY;
+  nsTArray<uint8_t> buffer;
+  buffer.SetLength(available);
   uint32_t numRead;
-  rv = stream->Read(TO_CHARBUFFER(buffer), available, &numRead);
+  rv = stream->Read(TO_CHARBUFFER(buffer.Elements()), available, &numRead);
   if (NS_FAILED(rv) || numRead != available) {
-    free(buffer);
     return rv;
   }
 
   nsAutoCString mimeType;
   rv = channel->GetContentType(mimeType);
   if (NS_FAILED(rv)) {
-    free(buffer);
     return rv;
   }
 
   // ReplaceFaviconData can now do the dirty work.
-  rv = ReplaceFaviconData(aFaviconURI, buffer, available, mimeType, aExpiration);
-  free(buffer);
+  rv = ReplaceFaviconData(aFaviconURI, buffer, mimeType, aExpiration);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFaviconService::GetFaviconURLForPage(nsIURI *aPageURI,
+nsFaviconService::GetFaviconURLForPage(nsIURI* aPageURI,
                                        nsIFaviconDataCallback* aCallback,
-                                       uint16_t aPreferredWidth)
-{
+                                       uint16_t aPreferredWidth) {
   MOZ_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG(aPageURI);
   NS_ENSURE_ARG(aCallback);
@@ -617,8 +570,8 @@ nsFaviconService::GetFaviconURLForPage(nsIURI *aPageURI,
   // It's expected that some domains may not have a host.
   Unused << aPageURI->GetHost(pageHost);
 
-  RefPtr<AsyncGetFaviconURLForPage> event =
-    new AsyncGetFaviconURLForPage(pageSpec, pageHost, aPreferredWidth, aCallback);
+  RefPtr<AsyncGetFaviconURLForPage> event = new AsyncGetFaviconURLForPage(
+      pageSpec, pageHost, aPreferredWidth, aCallback);
 
   RefPtr<Database> DB = Database::GetDatabase();
   NS_ENSURE_STATE(DB);
@@ -630,8 +583,7 @@ nsFaviconService::GetFaviconURLForPage(nsIURI *aPageURI,
 NS_IMETHODIMP
 nsFaviconService::GetFaviconDataForPage(nsIURI* aPageURI,
                                         nsIFaviconDataCallback* aCallback,
-                                        uint16_t aPreferredWidth)
-{
+                                        uint16_t aPreferredWidth) {
   MOZ_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG(aPageURI);
   NS_ENSURE_ARG(aCallback);
@@ -643,8 +595,8 @@ nsFaviconService::GetFaviconDataForPage(nsIURI* aPageURI,
   // It's expected that some domains may not have a host.
   Unused << aPageURI->GetHost(pageHost);
 
-  RefPtr<AsyncGetFaviconDataForPage> event =
-    new AsyncGetFaviconDataForPage(pageSpec, pageHost, aPreferredWidth, aCallback);
+  RefPtr<AsyncGetFaviconDataForPage> event = new AsyncGetFaviconDataForPage(
+      pageSpec, pageHost, aPreferredWidth, aCallback);
   RefPtr<Database> DB = Database::GetDatabase();
   NS_ENSURE_STATE(DB);
   DB->DispatchToAsyncThread(event);
@@ -653,17 +605,16 @@ nsFaviconService::GetFaviconDataForPage(nsIURI* aPageURI,
 }
 
 NS_IMETHODIMP
-nsFaviconService::CopyFavicons(nsIURI* aFromPageURI,
-                               nsIURI* aToPageURI,
+nsFaviconService::CopyFavicons(nsIURI* aFromPageURI, nsIURI* aToPageURI,
                                uint32_t aFaviconLoadType,
-                               nsIFaviconDataCallback* aCallback)
-{
+                               nsIFaviconDataCallback* aCallback) {
   MOZ_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG(aFromPageURI);
   NS_ENSURE_ARG(aToPageURI);
-  NS_ENSURE_TRUE(aFaviconLoadType >= nsIFaviconService::FAVICON_LOAD_PRIVATE &&
-                 aFaviconLoadType <= nsIFaviconService::FAVICON_LOAD_NON_PRIVATE,
-                 NS_ERROR_INVALID_ARG);
+  NS_ENSURE_TRUE(
+      aFaviconLoadType >= nsIFaviconService::FAVICON_LOAD_PRIVATE &&
+          aFaviconLoadType <= nsIFaviconService::FAVICON_LOAD_NON_PRIVATE,
+      NS_ERROR_INVALID_ARG);
 
   PageData fromPage;
   nsresult rv = aFromPageURI->GetSpec(fromPage.spec);
@@ -677,10 +628,12 @@ nsFaviconService::CopyFavicons(nsIURI* aFromPageURI,
   NS_ENSURE_TRUE(navHistory, NS_ERROR_OUT_OF_MEMORY);
   rv = navHistory->CanAddURI(aToPageURI, &canAddToHistory);
   NS_ENSURE_SUCCESS(rv, rv);
-  toPage.canAddToHistory = !!canAddToHistory &&
-                           aFaviconLoadType != nsIFaviconService::FAVICON_LOAD_PRIVATE;
+  toPage.canAddToHistory =
+      !!canAddToHistory &&
+      aFaviconLoadType != nsIFaviconService::FAVICON_LOAD_PRIVATE;
 
-  RefPtr<AsyncCopyFavicons> event = new AsyncCopyFavicons(fromPage, toPage, aCallback);
+  RefPtr<AsyncCopyFavicons> event =
+      new AsyncCopyFavicons(fromPage, toPage, aCallback);
 
   // Get the target thread and start the work.
   // DB will be updated and observers notified when done.
@@ -691,10 +644,8 @@ nsFaviconService::CopyFavicons(nsIURI* aFromPageURI,
   return NS_OK;
 }
 
-nsresult
-nsFaviconService::GetFaviconLinkForIcon(nsIURI* aFaviconURI,
-                                        nsIURI** aOutputURI)
-{
+nsresult nsFaviconService::GetFaviconLinkForIcon(nsIURI* aFaviconURI,
+                                                 nsIURI** aOutputURI) {
   NS_ENSURE_ARG(aFaviconURI);
   NS_ENSURE_ARG_POINTER(aOutputURI);
 
@@ -706,72 +657,13 @@ nsFaviconService::GetFaviconLinkForIcon(nsIURI* aFaviconURI,
   return GetFaviconLinkForIconString(spec, aOutputURI);
 }
 
-
-NS_IMETHODIMP
-nsFaviconService::AddFailedFavicon(nsIURI* aFaviconURI)
-{
-  NS_ENSURE_ARG(aFaviconURI);
-
-  nsAutoCString spec;
-  nsresult rv = aFaviconURI->GetSpec(spec);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  mFailedFavicons.Put(spec, mFailedFaviconSerial);
-  mFailedFaviconSerial ++;
-
-  if (mFailedFavicons.Count() > MAX_FAILED_FAVICONS) {
-    // need to expire some entries, delete the FAVICON_CACHE_REDUCE_COUNT number
-    // of items that are the oldest
-    uint32_t threshold = mFailedFaviconSerial -
-                         MAX_FAILED_FAVICONS + FAVICON_CACHE_REDUCE_COUNT;
-    for (auto iter = mFailedFavicons.Iter(); !iter.Done(); iter.Next()) {
-      if (iter.Data() < threshold) {
-        iter.Remove();
-      }
-    }
-  }
-  return NS_OK;
-}
-
-
-NS_IMETHODIMP
-nsFaviconService::RemoveFailedFavicon(nsIURI* aFaviconURI)
-{
-  NS_ENSURE_ARG(aFaviconURI);
-
-  nsAutoCString spec;
-  nsresult rv = aFaviconURI->GetSpec(spec);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // we silently do nothing and succeed if the icon is not in the cache
-  mFailedFavicons.Remove(spec);
-  return NS_OK;
-}
-
-
-NS_IMETHODIMP
-nsFaviconService::IsFailedFavicon(nsIURI* aFaviconURI, bool* _retval)
-{
-  NS_ENSURE_ARG(aFaviconURI);
-  nsAutoCString spec;
-  nsresult rv = aFaviconURI->GetSpec(spec);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  uint32_t serial;
-  *_retval = mFailedFavicons.Get(spec, &serial);
-  return NS_OK;
-}
-
-
 // nsFaviconService::GetFaviconLinkForIconString
 //
 //    This computes a favicon URL with string input and using the cached
 //    default one to minimize parsing.
 
-nsresult
-nsFaviconService::GetFaviconLinkForIconString(const nsCString& aSpec,
-                                              nsIURI** aOutput)
-{
+nsresult nsFaviconService::GetFaviconLinkForIconString(const nsCString& aSpec,
+                                                       nsIURI** aOutput) {
   if (aSpec.IsEmpty()) {
     return GetDefaultFavicon(aOutput);
   }
@@ -794,9 +686,7 @@ nsFaviconService::GetFaviconLinkForIconString(const nsCString& aSpec,
  * @param aIcon
  *        The icon to be evaluated.
  */
-nsresult
-nsFaviconService::OptimizeIconSizes(IconData& aIcon)
-{
+nsresult nsFaviconService::OptimizeIconSizes(IconData& aIcon) {
   // TODO (bug 1346139): move optimization to the async thread.
   MOZ_ASSERT(NS_IsMainThread());
   // There should only be a single payload at this point, it may have to be
@@ -820,10 +710,9 @@ nsFaviconService::OptimizeIconSizes(IconData& aIcon)
 
   // decode image
   nsCOMPtr<imgIContainer> container;
-  nsresult rv = GetImgTools()->DecodeImageFromBuffer(payload.data.get(),
-                                                     payload.data.Length(),
-                                                     payload.mimeType,
-                                                     getter_AddRefs(container));
+  nsresult rv = GetImgTools()->DecodeImageFromBuffer(
+      payload.data.get(), payload.data.Length(), payload.mimeType,
+      getter_AddRefs(container));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // For ICO files, we must evaluate each of the frames we care about.
@@ -835,7 +724,7 @@ nsFaviconService::OptimizeIconSizes(IconData& aIcon)
     IconPayload newPayload;
     newPayload.mimeType = NS_LITERAL_CSTRING(PNG_MIME_TYPE);
     newPayload.width = frameInfo.width;
-    for (uint16_t size : sFaviconSizes) {
+    for (uint16_t size : gFaviconSizes) {
       // The icon could be smaller than 16, that is our minimum.
       // Icons smaller than 16px are kept as-is.
       if (frameInfo.width >= 16) {
@@ -853,14 +742,12 @@ nsFaviconService::OptimizeIconSizes(IconData& aIcon)
         newPayload.data = payload.data;
       } else {
         // Otherwise, scale and recompress.
-        // Since EncodeScaledImage uses SYNC_DECODE, it will pick the best frame.
+        // Since EncodeScaledImage uses SYNC_DECODE, it will pick the best
+        // frame.
         nsCOMPtr<nsIInputStream> iconStream;
-        rv = GetImgTools()->EncodeScaledImage(container,
-                                              newPayload.mimeType,
-                                              newPayload.width,
-                                              newPayload.width,
-                                              EmptyString(),
-                                              getter_AddRefs(iconStream));
+        rv = GetImgTools()->EncodeScaledImage(
+            container, newPayload.mimeType, newPayload.width, newPayload.width,
+            EmptyString(), getter_AddRefs(iconStream));
         NS_ENSURE_SUCCESS(rv, rv);
         // Read the stream into the new buffer.
         rv = NS_ConsumeStream(iconStream, UINT32_MAX, newPayload.data);
@@ -868,12 +755,14 @@ nsFaviconService::OptimizeIconSizes(IconData& aIcon)
       }
 
       // If the icon size is good, we are done, otherwise try the next size.
-      if (newPayload.data.Length() < nsIFaviconService::MAX_FAVICON_BUFFER_SIZE) {
+      if (newPayload.data.Length() <
+          nsIFaviconService::MAX_FAVICON_BUFFER_SIZE) {
         break;
       }
     }
 
-    MOZ_ASSERT(newPayload.data.Length() < nsIFaviconService::MAX_FAVICON_BUFFER_SIZE);
+    MOZ_ASSERT(newPayload.data.Length() <
+               nsIFaviconService::MAX_FAVICON_BUFFER_SIZE);
     if (newPayload.data.Length() < nsIFaviconService::MAX_FAVICON_BUFFER_SIZE) {
       aIcon.payloads.AppendElement(newPayload);
     }
@@ -882,18 +771,15 @@ nsFaviconService::OptimizeIconSizes(IconData& aIcon)
   return NS_OK;
 }
 
-nsresult
-nsFaviconService::GetFaviconDataAsync(const nsCString& aFaviconURI,
-                                      mozIStorageStatementCallback *aCallback)
-{
+nsresult nsFaviconService::GetFaviconDataAsync(
+    const nsCString& aFaviconURI, mozIStorageStatementCallback* aCallback) {
   MOZ_ASSERT(aCallback, "Doesn't make sense to call this without a callback");
 
   nsCOMPtr<mozIStorageAsyncStatement> stmt = mDB->GetAsyncStatement(
-    "/*Do not warn (bug no: not worth adding an index */ "
-    "SELECT data, width FROM moz_icons "
-    "WHERE fixed_icon_url_hash = hash(fixup_url(:url)) AND icon_url = :url "
-    "ORDER BY width DESC"
-  );
+      "/*Do not warn (bug no: not worth adding an index */ "
+      "SELECT data, width FROM moz_icons "
+      "WHERE fixed_icon_url_hash = hash(fixup_url(:url)) AND icon_url = :url "
+      "ORDER BY width DESC");
   NS_ENSURE_STATE(stmt);
 
   nsresult rv = URIBinder::Bind(stmt, NS_LITERAL_CSTRING("url"), aFaviconURI);
@@ -903,19 +789,19 @@ nsFaviconService::GetFaviconDataAsync(const nsCString& aFaviconURI,
   return stmt->ExecuteAsync(aCallback, getter_AddRefs(pendingStatement));
 }
 
-void // static
-nsFaviconService::ConvertUnsupportedPayloads(mozIStorageConnection* aDBConn)
-{
+void  // static
+nsFaviconService::ConvertUnsupportedPayloads(mozIStorageConnection* aDBConn) {
   MOZ_ASSERT(NS_IsMainThread());
   // Ensure imgTools are initialized, so that the image decoders can be used
   // off the main thread.
-  nsCOMPtr<imgITools> imgTools = do_CreateInstance("@mozilla.org/image/tools;1");
+  nsCOMPtr<imgITools> imgTools =
+      do_CreateInstance("@mozilla.org/image/tools;1");
 
   Preferences::SetBool(PREF_CONVERT_PAYLOADS, true);
   MOZ_ASSERT(aDBConn);
   if (aDBConn) {
     RefPtr<FetchAndConvertUnsupportedPayloads> event =
-      new FetchAndConvertUnsupportedPayloads(aDBConn);
+        new FetchAndConvertUnsupportedPayloads(aDBConn);
     nsCOMPtr<nsIEventTarget> target = do_GetInterface(aDBConn);
     MOZ_ASSERT(target);
     if (target) {
@@ -931,13 +817,11 @@ nsFaviconService::SetDefaultIconURIPreferredSize(uint16_t aDefaultSize) {
 }
 
 NS_IMETHODIMP
-nsFaviconService::PreferredSizeFromURI(nsIURI* aURI, uint16_t* _size)
-{
+nsFaviconService::PreferredSizeFromURI(nsIURI* aURI, uint16_t* _size) {
   *_size = mDefaultIconURIPreferredSize;
   nsAutoCString ref;
   // Check for a ref first.
-  if (NS_FAILED(aURI->GetRef(ref)) || ref.Length() == 0)
-    return NS_OK;
+  if (NS_FAILED(aURI->GetRef(ref)) || ref.Length() == 0) return NS_OK;
 
   // Look for a "size=" fragment.
   int32_t start = ref.RFind("size=");
@@ -966,24 +850,19 @@ nsFaviconService::PreferredSizeFromURI(nsIURI* aURI, uint16_t* _size)
 ////////////////////////////////////////////////////////////////////////////////
 //// ExpireFaviconsStatementCallbackNotifier
 
-ExpireFaviconsStatementCallbackNotifier::ExpireFaviconsStatementCallbackNotifier()
-{
-}
-
+ExpireFaviconsStatementCallbackNotifier::
+    ExpireFaviconsStatementCallbackNotifier() {}
 
 NS_IMETHODIMP
-ExpireFaviconsStatementCallbackNotifier::HandleCompletion(uint16_t aReason)
-{
+ExpireFaviconsStatementCallbackNotifier::HandleCompletion(uint16_t aReason) {
   // We should dispatch only if expiration has been successful.
-  if (aReason != mozIStorageStatementCallback::REASON_FINISHED)
-    return NS_OK;
+  if (aReason != mozIStorageStatementCallback::REASON_FINISHED) return NS_OK;
 
   nsCOMPtr<nsIObserverService> observerService =
-    mozilla::services::GetObserverService();
+      mozilla::services::GetObserverService();
   if (observerService) {
-    (void)observerService->NotifyObservers(nullptr,
-                                           NS_PLACES_FAVICONS_EXPIRED_TOPIC_ID,
-                                           nullptr);
+    (void)observerService->NotifyObservers(
+        nullptr, NS_PLACES_FAVICONS_EXPIRED_TOPIC_ID, nullptr);
   }
 
   return NS_OK;
