@@ -46,7 +46,11 @@
 
 #if defined(XP_WIN)
 // Prevent collisions with nsAppStartup::GetStartupInfo()
-#undef GetStartupInfo
+#  undef GetStartupInfo
+
+#  include <windows.h>
+#elif defined(XP_DARWIN)
+#  include <mach/mach_time.h>
 #endif
 
 #include "mozilla/IOInterposer.h"
@@ -60,56 +64,64 @@ static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 #define kPrefRecentCrashes "toolkit.startup.recent_crashes"
 #define kPrefAlwaysUseSafeMode "toolkit.startup.always_use_safe_mode"
 
+#define kNanosecondsPerSecond 1000000000.0
+
 #if defined(XP_WIN)
-#include "mozilla/perfprobe.h"
+#  include "mozilla/perfprobe.h"
 /**
  * Events sent to the system for profiling purposes
  */
-//Keep them syncronized with the .mof file
+// Keep them syncronized with the .mof file
 
-//Process-wide GUID, used by the OS to differentiate sources
+// Process-wide GUID, used by the OS to differentiate sources
 // {509962E0-406B-46F4-99BA-5A009F8D2225}
-//Keep it synchronized with the .mof file
-#define NS_APPLICATION_TRACING_CID \
-  { 0x509962E0, 0x406B, 0x46F4, \
-  { 0x99, 0xBA, 0x5A, 0x00, 0x9F, 0x8D, 0x22, 0x25} }
+// Keep it synchronized with the .mof file
+#  define NS_APPLICATION_TRACING_CID                   \
+    {                                                  \
+      0x509962E0, 0x406B, 0x46F4, {                    \
+        0x99, 0xBA, 0x5A, 0x00, 0x9F, 0x8D, 0x22, 0x25 \
+      }                                                \
+    }
 
-//Event-specific GUIDs, used by the OS to differentiate events
+// Event-specific GUIDs, used by the OS to differentiate events
 // {A3DA04E0-57D7-482A-A1C1-61DA5F95BACB}
-#define NS_PLACES_INIT_COMPLETE_EVENT_CID \
-  { 0xA3DA04E0, 0x57D7, 0x482A, \
-  { 0xA1, 0xC1, 0x61, 0xDA, 0x5F, 0x95, 0xBA, 0xCB} }
+#  define NS_PLACES_INIT_COMPLETE_EVENT_CID            \
+    {                                                  \
+      0xA3DA04E0, 0x57D7, 0x482A, {                    \
+        0xA1, 0xC1, 0x61, 0xDA, 0x5F, 0x95, 0xBA, 0xCB \
+      }                                                \
+    }
 // {917B96B1-ECAD-4DAB-A760-8D49027748AE}
-#define NS_SESSION_STORE_WINDOW_RESTORED_EVENT_CID \
-  { 0x917B96B1, 0xECAD, 0x4DAB, \
-  { 0xA7, 0x60, 0x8D, 0x49, 0x02, 0x77, 0x48, 0xAE} }
+#  define NS_SESSION_STORE_WINDOW_RESTORED_EVENT_CID   \
+    {                                                  \
+      0x917B96B1, 0xECAD, 0x4DAB, {                    \
+        0xA7, 0x60, 0x8D, 0x49, 0x02, 0x77, 0x48, 0xAE \
+      }                                                \
+    }
 // {26D1E091-0AE7-4F49-A554-4214445C505C}
-#define NS_XPCOM_SHUTDOWN_EVENT_CID \
-  { 0x26D1E091, 0x0AE7, 0x4F49, \
-  { 0xA5, 0x54, 0x42, 0x14, 0x44, 0x5C, 0x50, 0x5C} }
+#  define NS_XPCOM_SHUTDOWN_EVENT_CID                  \
+    {                                                  \
+      0x26D1E091, 0x0AE7, 0x4F49, {                    \
+        0xA5, 0x54, 0x42, 0x14, 0x44, 0x5C, 0x50, 0x5C \
+      }                                                \
+    }
 
-static NS_DEFINE_CID(kApplicationTracingCID,
-  NS_APPLICATION_TRACING_CID);
-static NS_DEFINE_CID(kPlacesInitCompleteCID,
-  NS_PLACES_INIT_COMPLETE_EVENT_CID);
+static NS_DEFINE_CID(kApplicationTracingCID, NS_APPLICATION_TRACING_CID);
+static NS_DEFINE_CID(kPlacesInitCompleteCID, NS_PLACES_INIT_COMPLETE_EVENT_CID);
 static NS_DEFINE_CID(kSessionStoreWindowRestoredCID,
-  NS_SESSION_STORE_WINDOW_RESTORED_EVENT_CID);
-static NS_DEFINE_CID(kXPCOMShutdownCID,
-  NS_XPCOM_SHUTDOWN_EVENT_CID);
-#endif //defined(XP_WIN)
+                     NS_SESSION_STORE_WINDOW_RESTORED_EVENT_CID);
+static NS_DEFINE_CID(kXPCOMShutdownCID, NS_XPCOM_SHUTDOWN_EVENT_CID);
+#endif  // defined(XP_WIN)
 
 using namespace mozilla;
 
 class nsAppExitEvent : public mozilla::Runnable {
-private:
+ private:
   RefPtr<nsAppStartup> mService;
 
-public:
+ public:
   explicit nsAppExitEvent(nsAppStartup* service)
-    : mozilla::Runnable("nsAppExitEvent")
-    , mService(service)
-  {
-  }
+      : mozilla::Runnable("nsAppExitEvent"), mService(service) {}
 
   NS_IMETHOD Run() override {
     // Tell the appshell to exit
@@ -129,9 +141,7 @@ public:
  * @param stamp The timestamp to be converted
  * @returns The converted timestamp
  */
-static uint64_t
-ComputeAbsoluteTimestamp(TimeStamp stamp)
-{
+static uint64_t ComputeAbsoluteTimestamp(TimeStamp stamp) {
   static PRTime sAbsoluteNow = PR_Now();
   static TimeStamp sMonotonicNow = TimeStamp::Now();
 
@@ -142,33 +152,26 @@ ComputeAbsoluteTimestamp(TimeStamp stamp)
 // nsAppStartup
 //
 
-nsAppStartup::nsAppStartup() :
-  mConsiderQuitStopper(0),
-  mRunning(false),
-  mShuttingDown(false),
-  mStartingUp(true),
-  mAttemptingQuit(false),
-  mRestart(false),
-  mInterrupted(false),
-  mIsSafeModeNecessary(false),
-  mStartupCrashTrackingEnded(false),
-  mRestartNotSameProfile(false)
-{ }
+nsAppStartup::nsAppStartup()
+    : mConsiderQuitStopper(0),
+      mRunning(false),
+      mShuttingDown(false),
+      mStartingUp(true),
+      mAttemptingQuit(false),
+      mRestart(false),
+      mInterrupted(false),
+      mIsSafeModeNecessary(false),
+      mStartupCrashTrackingEnded(false) {}
 
-
-nsresult
-nsAppStartup::Init()
-{
+nsresult nsAppStartup::Init() {
   nsresult rv;
 
   // Create widget application shell
   mAppShell = do_GetService(kAppShellCID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIObserverService> os =
-    mozilla::services::GetObserverService();
-  if (!os)
-    return NS_ERROR_FAILURE;
+  nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
+  if (!os) return NS_ERROR_FAILURE;
 
   os->AddObserver(this, "quit-application", true);
   os->AddObserver(this, "quit-application-forced", true);
@@ -185,33 +188,26 @@ nsAppStartup::Init()
   // This last event is only interesting to us for xperf-based measures
 
   // Initialize interaction with profiler
-  mProbesManager =
-    new ProbeManager(
-                     kApplicationTracingCID,
-                     NS_LITERAL_CSTRING("Application startup probe"));
+  mProbesManager = new ProbeManager(
+      kApplicationTracingCID, NS_LITERAL_CSTRING("Application startup probe"));
   // Note: The operation is meant mostly for in-house profiling.
   // Therefore, we do not warn if probes manager cannot be initialized
 
   if (mProbesManager) {
-    mPlacesInitCompleteProbe =
-      mProbesManager->
-      GetProbe(kPlacesInitCompleteCID,
-               NS_LITERAL_CSTRING("places-init-complete"));
+    mPlacesInitCompleteProbe = mProbesManager->GetProbe(
+        kPlacesInitCompleteCID, NS_LITERAL_CSTRING("places-init-complete"));
     NS_WARNING_ASSERTION(mPlacesInitCompleteProbe,
                          "Cannot initialize probe 'places-init-complete'");
 
-    mSessionWindowRestoredProbe =
-      mProbesManager->
-      GetProbe(kSessionStoreWindowRestoredCID,
-               NS_LITERAL_CSTRING("sessionstore-windows-restored"));
+    mSessionWindowRestoredProbe = mProbesManager->GetProbe(
+        kSessionStoreWindowRestoredCID,
+        NS_LITERAL_CSTRING("sessionstore-windows-restored"));
     NS_WARNING_ASSERTION(
-      mSessionWindowRestoredProbe,
-      "Cannot initialize probe 'sessionstore-windows-restored'");
+        mSessionWindowRestoredProbe,
+        "Cannot initialize probe 'sessionstore-windows-restored'");
 
-    mXPCOMShutdownProbe =
-      mProbesManager->
-      GetProbe(kXPCOMShutdownCID,
-               NS_LITERAL_CSTRING("xpcom-shutdown"));
+    mXPCOMShutdownProbe = mProbesManager->GetProbe(
+        kXPCOMShutdownCID, NS_LITERAL_CSTRING("xpcom-shutdown"));
     NS_WARNING_ASSERTION(mXPCOMShutdownProbe,
                          "Cannot initialize probe 'xpcom-shutdown'");
 
@@ -219,51 +215,42 @@ nsAppStartup::Init()
     NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                          "Cannot initialize system probe manager");
   }
-#endif //defined(XP_WIN)
+#endif  // defined(XP_WIN)
 
   return NS_OK;
 }
 
-
 //
 // nsAppStartup->nsISupports
 //
 
-NS_IMPL_ISUPPORTS(nsAppStartup,
-                  nsIAppStartup,
-                  nsIWindowCreator,
-                  nsIWindowCreator2,
-                  nsIObserver,
-                  nsISupportsWeakReference)
-
+NS_IMPL_ISUPPORTS(nsAppStartup, nsIAppStartup, nsIWindowCreator,
+                  nsIWindowCreator2, nsIObserver, nsISupportsWeakReference)
 
 //
 // nsAppStartup->nsIAppStartup
 //
 
 NS_IMETHODIMP
-nsAppStartup::CreateHiddenWindow()
-{
+nsAppStartup::CreateHiddenWindow() {
 #if defined(MOZ_WIDGET_UIKIT)
   return NS_OK;
 #else
-  nsCOMPtr<nsIAppShellService> appShellService
-    (do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
+  nsCOMPtr<nsIAppShellService> appShellService(
+      do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
   NS_ENSURE_TRUE(appShellService, NS_ERROR_FAILURE);
 
   return appShellService->CreateHiddenWindow();
 #endif
 }
 
-
 NS_IMETHODIMP
-nsAppStartup::DestroyHiddenWindow()
-{
+nsAppStartup::DestroyHiddenWindow() {
 #if defined(MOZ_WIDGET_UIKIT)
   return NS_OK;
 #else
-  nsCOMPtr<nsIAppShellService> appShellService
-    (do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
+  nsCOMPtr<nsIAppShellService> appShellService(
+      do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
   NS_ENSURE_TRUE(appShellService, NS_ERROR_FAILURE);
 
   return appShellService->DestroyHiddenWindow();
@@ -271,8 +258,7 @@ nsAppStartup::DestroyHiddenWindow()
 }
 
 NS_IMETHODIMP
-nsAppStartup::Run(void)
-{
+nsAppStartup::Run(void) {
   NS_ASSERTION(!mRunning, "Reentrant appstartup->Run()");
 
   // If we have no windows open and no explicit calls to
@@ -288,25 +274,24 @@ nsAppStartup::Run(void)
     mRunning = true;
 
     nsresult rv = mAppShell->Run();
-    if (NS_FAILED(rv))
-      return rv;
+    if (NS_FAILED(rv)) return rv;
   }
 
+  // Make sure that the appropriate quit notifications have been dispatched
+  // regardless of whether the event loop has spun or not. Note that this call
+  // is a no-op if Quit has already been called previously.
+  Quit(eForceQuit);
+
   nsresult retval = NS_OK;
   if (mRestart) {
     retval = NS_SUCCESS_RESTART_APP;
-  } else if (mRestartNotSameProfile) {
-    retval = NS_SUCCESS_RESTART_APP_NOT_SAME_PROFILE;
   }
 
   return retval;
 }
 
-
-
 NS_IMETHODIMP
-nsAppStartup::Quit(uint32_t aMode)
-{
+nsAppStartup::Quit(uint32_t aMode) {
   uint32_t ferocity = (aMode & 0xF);
 
   // Quit the application. We will asynchronously call the appshell's
@@ -315,14 +300,13 @@ nsAppStartup::Quit(uint32_t aMode)
   nsresult rv = NS_OK;
   bool postedExitEvent = false;
 
-  if (mShuttingDown)
-    return NS_OK;
+  if (mShuttingDown) return NS_OK;
 
   // If we're considering quitting, we will only do so if:
   if (ferocity == eConsiderQuit) {
 #ifdef XP_MACOSX
-    nsCOMPtr<nsIAppShellService> appShell
-      (do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
+    nsCOMPtr<nsIAppShellService> appShell(
+        do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
     bool hasHiddenPrivateWindow = false;
     if (appShell) {
       appShell->GetHasHiddenPrivateWindow(&hasHiddenPrivateWindow);
@@ -339,8 +323,7 @@ nsAppStartup::Quit(uint32_t aMode)
       // ... or there is only a hiddenWindow left, and it's useless:
 
       // Failure shouldn't be fatal, but will abort quit attempt:
-      if (!appShell)
-        return NS_OK;
+      if (!appShell) return NS_OK;
 
       bool usefulHiddenWindow;
       appShell->GetApplicationProvidedHiddenWindow(&usefulHiddenWindow);
@@ -363,9 +346,9 @@ nsAppStartup::Quit(uint32_t aMode)
 
   nsCOMPtr<nsIObserverService> obsService;
   if (ferocity == eAttemptQuit || ferocity == eForceQuit) {
-
     nsCOMPtr<nsISimpleEnumerator> windowEnumerator;
-    nsCOMPtr<nsIWindowMediator> mediator (do_GetService(NS_WINDOWMEDIATOR_CONTRACTID));
+    nsCOMPtr<nsIWindowMediator> mediator(
+        do_GetService(NS_WINDOWMEDIATOR_CONTRACTID));
     if (mediator) {
       mediator->GetEnumerator(nullptr, getter_AddRefs(windowEnumerator));
       if (windowEnumerator) {
@@ -380,26 +363,21 @@ nsAppStartup::Quit(uint32_t aMode)
           windowEnumerator->GetNext(getter_AddRefs(window));
           nsCOMPtr<nsPIDOMWindowOuter> domWindow(do_QueryInterface(window));
           if (domWindow) {
-            if (!domWindow->CanClose())
-              return NS_OK;
+            if (!domWindow->CanClose()) return NS_OK;
           }
           windowEnumerator->HasMoreElements(&more);
         }
       }
     }
 
-    PROFILER_ADD_MARKER("Shutdown start");
+    PROFILER_ADD_MARKER("Shutdown start", OTHER);
     mozilla::RecordShutdownStartTimeStamp();
     mShuttingDown = true;
     if (!mRestart) {
       mRestart = (aMode & eRestart) != 0;
     }
 
-    if (!mRestartNotSameProfile) {
-      mRestartNotSameProfile = (aMode & eRestartNotSameProfile) != 0;
-    }
-
-    if (mRestart || mRestartNotSameProfile) {
+    if (mRestart) {
       // Mark the next startup as a restart.
       PR_SetEnv("MOZ_APP_RESTART=1");
 
@@ -417,7 +395,8 @@ nsAppStartup::Quit(uint32_t aMode)
       ExitLastWindowClosingSurvivalArea();
 #endif
       if (obsService)
-        obsService->NotifyObservers(nullptr, "quit-application-granted", nullptr);
+        obsService->NotifyObservers(nullptr, "quit-application-granted",
+                                    nullptr);
     }
 
     /* Enumerate through each open window and close it. It's important to do
@@ -428,7 +407,7 @@ nsAppStartup::Quit(uint32_t aMode)
 
     if (mediator) {
       if (ferocity == eAttemptQuit) {
-        ferocity = eForceQuit; // assume success
+        ferocity = eForceQuit;  // assume success
 
         /* Were we able to immediately close all windows? if not, eAttemptQuit
            failed. This could happen for a variety of reasons; in fact it's
@@ -466,13 +445,12 @@ nsAppStartup::Quit(uint32_t aMode)
     // we're shutting down for sure while all services are still available.
     if (obsService) {
       obsService->NotifyObservers(nullptr, "quit-application",
-        (mRestart || mRestartNotSameProfile) ? u"restart" : u"shutdown");
+                                  mRestart ? u"restart" : u"shutdown");
     }
 
     if (!mRunning) {
       postedExitEvent = true;
-    }
-    else {
+    } else {
       // no matter what, make sure we send the exit event.  If
       // worst comes to worst, we'll do a leaky shutdown but we WILL
       // shut down. Well, assuming that all *this* stuff works ;-).
@@ -480,8 +458,7 @@ nsAppStartup::Quit(uint32_t aMode)
       rv = NS_DispatchToCurrentThread(event);
       if (NS_SUCCEEDED(rv)) {
         postedExitEvent = true;
-      }
-      else {
+      } else {
         NS_WARNING("failed to dispatch nsAppExitEvent");
       }
     }
@@ -489,30 +466,24 @@ nsAppStartup::Quit(uint32_t aMode)
 
   // turn off the reentrancy check flag, but not if we have
   // more asynchronous work to do still.
-  if (!postedExitEvent)
-    mShuttingDown = false;
+  if (!postedExitEvent) mShuttingDown = false;
   return rv;
 }
 
-
-void
-nsAppStartup::CloseAllWindows()
-{
-  nsCOMPtr<nsIWindowMediator> mediator
-    (do_GetService(NS_WINDOWMEDIATOR_CONTRACTID));
+void nsAppStartup::CloseAllWindows() {
+  nsCOMPtr<nsIWindowMediator> mediator(
+      do_GetService(NS_WINDOWMEDIATOR_CONTRACTID));
 
   nsCOMPtr<nsISimpleEnumerator> windowEnumerator;
 
   mediator->GetEnumerator(nullptr, getter_AddRefs(windowEnumerator));
 
-  if (!windowEnumerator)
-    return;
+  if (!windowEnumerator) return;
 
   bool more;
   while (NS_SUCCEEDED(windowEnumerator->HasMoreElements(&more)) && more) {
     nsCOMPtr<nsISupports> isupports;
-    if (NS_FAILED(windowEnumerator->GetNext(getter_AddRefs(isupports))))
-      break;
+    if (NS_FAILED(windowEnumerator->GetNext(getter_AddRefs(isupports)))) break;
 
     nsCOMPtr<nsPIDOMWindowOuter> window = do_QueryInterface(isupports);
     NS_ASSERTION(window, "not an nsPIDOMWindow");
@@ -523,21 +494,17 @@ nsAppStartup::CloseAllWindows()
 }
 
 NS_IMETHODIMP
-nsAppStartup::EnterLastWindowClosingSurvivalArea(void)
-{
+nsAppStartup::EnterLastWindowClosingSurvivalArea(void) {
   ++mConsiderQuitStopper;
   return NS_OK;
 }
 
-
 NS_IMETHODIMP
-nsAppStartup::ExitLastWindowClosingSurvivalArea(void)
-{
+nsAppStartup::ExitLastWindowClosingSurvivalArea(void) {
   NS_ASSERTION(mConsiderQuitStopper > 0, "consider quit stopper out of bounds");
   --mConsiderQuitStopper;
 
-  if (mRunning)
-    Quit(eConsiderQuit);
+  if (mRunning) Quit(eConsiderQuit);
 
   return NS_OK;
 }
@@ -547,22 +514,19 @@ nsAppStartup::ExitLastWindowClosingSurvivalArea(void)
 //
 
 NS_IMETHODIMP
-nsAppStartup::GetShuttingDown(bool *aResult)
-{
+nsAppStartup::GetShuttingDown(bool* aResult) {
   *aResult = mShuttingDown;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsAppStartup::GetStartingUp(bool *aResult)
-{
+nsAppStartup::GetStartingUp(bool* aResult) {
   *aResult = mStartingUp;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsAppStartup::DoneStartingUp()
-{
+nsAppStartup::DoneStartingUp() {
   // This must be called once at most
   MOZ_ASSERT(mStartingUp);
 
@@ -571,16 +535,14 @@ nsAppStartup::DoneStartingUp()
 }
 
 NS_IMETHODIMP
-nsAppStartup::GetRestarting(bool *aResult)
-{
+nsAppStartup::GetRestarting(bool* aResult) {
   *aResult = mRestart;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsAppStartup::GetWasRestarted(bool *aResult)
-{
-  char *mozAppRestart = PR_GetEnv("MOZ_APP_RESTART");
+nsAppStartup::GetWasRestarted(bool* aResult) {
+  char* mozAppRestart = PR_GetEnv("MOZ_APP_RESTART");
 
   /* When calling PR_SetEnv() with an empty value the existing variable may
    * be unset or set to the empty string depending on the underlying platform
@@ -591,15 +553,32 @@ nsAppStartup::GetWasRestarted(bool *aResult)
 }
 
 NS_IMETHODIMP
-nsAppStartup::SetInterrupted(bool aInterrupted)
-{
+nsAppStartup::GetSecondsSinceLastOSRestart(int64_t *aResult) {
+#if defined(XP_WIN)
+  *aResult = int64_t(GetTickCount64() / 1000ull);
+  return NS_OK;
+#elif defined(XP_DARWIN)
+  uint64_t absTime = mach_absolute_time();
+  mach_timebase_info_data_t timebaseInfo;
+  mach_timebase_info(&timebaseInfo);
+  double toNanoseconds =
+      double(timebaseInfo.numer) / double(timebaseInfo.denom);
+  *aResult =
+      std::llround(double(absTime) * toNanoseconds / kNanosecondsPerSecond);
+  return NS_OK;
+#else
+  return NS_ERROR_NOT_IMPLEMENTED;
+#endif
+}
+
+NS_IMETHODIMP
+nsAppStartup::SetInterrupted(bool aInterrupted) {
   mInterrupted = aInterrupted;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsAppStartup::GetInterrupted(bool *aInterrupted)
-{
+nsAppStartup::GetInterrupted(bool* aInterrupted) {
   *aInterrupted = mInterrupted;
   return NS_OK;
 }
@@ -609,23 +588,22 @@ nsAppStartup::GetInterrupted(bool *aInterrupted)
 //
 
 NS_IMETHODIMP
-nsAppStartup::CreateChromeWindow(nsIWebBrowserChrome *aParent,
+nsAppStartup::CreateChromeWindow(nsIWebBrowserChrome* aParent,
                                  uint32_t aChromeFlags,
-                                 nsIWebBrowserChrome **_retval)
-{
+                                 nsIWebBrowserChrome** _retval) {
   bool cancel;
-  return CreateChromeWindow2(aParent, aChromeFlags, nullptr, nullptr, 0, &cancel, _retval);
+  return CreateChromeWindow2(aParent, aChromeFlags, nullptr, nullptr, 0,
+                             &cancel, _retval);
 }
 
-
 //
 // nsAppStartup->nsIWindowCreator2
 //
 
 NS_IMETHODIMP
-nsAppStartup::SetScreenId(uint32_t aScreenId)
-{
-  nsCOMPtr<nsIAppShellService> appShell(do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
+nsAppStartup::SetScreenId(uint32_t aScreenId) {
+  nsCOMPtr<nsIAppShellService> appShell(
+      do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
   if (!appShell) {
     return NS_ERROR_FAILURE;
   }
@@ -634,38 +612,46 @@ nsAppStartup::SetScreenId(uint32_t aScreenId)
 }
 
 NS_IMETHODIMP
-nsAppStartup::CreateChromeWindow2(nsIWebBrowserChrome *aParent,
+nsAppStartup::CreateChromeWindow2(nsIWebBrowserChrome* aParent,
                                   uint32_t aChromeFlags,
-                                  nsITabParent *aOpeningTab,
+                                  nsIRemoteTab* aOpeningTab,
                                   mozIDOMWindowProxy* aOpener,
-                                  uint64_t aNextTabParentId,
-                                  bool *aCancel,
-                                  nsIWebBrowserChrome **_retval)
-{
+                                  uint64_t aNextRemoteTabId, bool* aCancel,
+                                  nsIWebBrowserChrome** _retval) {
   NS_ENSURE_ARG_POINTER(aCancel);
   NS_ENSURE_ARG_POINTER(_retval);
   *aCancel = false;
   *_retval = 0;
 
   // Non-modal windows cannot be opened if we are attempting to quit
-  if (mAttemptingQuit && (aChromeFlags & nsIWebBrowserChrome::CHROME_MODAL) == 0)
+  if (mAttemptingQuit &&
+      (aChromeFlags & nsIWebBrowserChrome::CHROME_MODAL) == 0)
     return NS_ERROR_ILLEGAL_DURING_SHUTDOWN;
 
+  // Fission windows must also be marked as remote
+  if ((aChromeFlags & nsIWebBrowserChrome::CHROME_FISSION_WINDOW) &&
+      !(aChromeFlags & nsIWebBrowserChrome::CHROME_REMOTE_WINDOW)) {
+    NS_WARNING("Cannot create non-remote fission window!");
+    return NS_ERROR_FAILURE;
+  }
+
   nsCOMPtr<nsIXULWindow> newWindow;
 
   if (aParent) {
     nsCOMPtr<nsIXULWindow> xulParent(do_GetInterface(aParent));
-    NS_ASSERTION(xulParent, "window created using non-XUL parent. that's unexpected, but may work.");
+    NS_ASSERTION(xulParent,
+                 "window created using non-XUL parent. that's unexpected, but "
+                 "may work.");
 
     if (xulParent)
       xulParent->CreateNewWindow(aChromeFlags, aOpeningTab, aOpener,
-                                 aNextTabParentId,
-                                 getter_AddRefs(newWindow));
+                                 aNextRemoteTabId, getter_AddRefs(newWindow));
     // And if it fails, don't try again without a parent. It could fail
     // intentionally (bug 115969).
-  } else { // try using basic methods:
-    MOZ_RELEASE_ASSERT(aNextTabParentId == 0,
-                       "Unexpected aNextTabParentId, we shouldn't ever have a next actor ID without a parent");
+  } else {  // try using basic methods:
+    MOZ_RELEASE_ASSERT(aNextRemoteTabId == 0,
+                       "Unexpected aNextRemoteTabId, we shouldn't ever have a "
+                       "next actor ID without a parent");
 
     /* You really shouldn't be making dependent windows without a parent.
       But unparented modal (and therefore dependent) windows happen
@@ -673,41 +659,36 @@ nsAppStartup::CreateChromeWindow2(nsIWebBrowserChrome *aParent,
     if (aChromeFlags & nsIWebBrowserChrome::CHROME_DEPENDENT)
       NS_WARNING("dependent window created without a parent");
 
-    nsCOMPtr<nsIAppShellService> appShell(do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
-    if (!appShell)
-      return NS_ERROR_FAILURE;
+    nsCOMPtr<nsIAppShellService> appShell(
+        do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
+    if (!appShell) return NS_ERROR_FAILURE;
 
-    appShell->CreateTopLevelWindow(0, 0, aChromeFlags,
-                                   nsIAppShellService::SIZE_TO_CONTENT,
-                                   nsIAppShellService::SIZE_TO_CONTENT,
-                                   aOpeningTab, aOpener,
-                                   getter_AddRefs(newWindow));
+    appShell->CreateTopLevelWindow(
+        0, 0, aChromeFlags, nsIAppShellService::SIZE_TO_CONTENT,
+        nsIAppShellService::SIZE_TO_CONTENT, aOpeningTab, aOpener,
+        getter_AddRefs(newWindow));
   }
 
   // if anybody gave us anything to work with, use it
   if (newWindow) {
     nsCOMPtr<nsIInterfaceRequestor> thing(do_QueryInterface(newWindow));
-    if (thing)
-      CallGetInterface(thing.get(), _retval);
+    if (thing) CallGetInterface(thing.get(), _retval);
   }
 
   return *_retval ? NS_OK : NS_ERROR_FAILURE;
 }
 
-
 //
 // nsAppStartup->nsIObserver
 //
 
 NS_IMETHODIMP
-nsAppStartup::Observe(nsISupports *aSubject,
-                      const char *aTopic, const char16_t *aData)
-{
+nsAppStartup::Observe(nsISupports* aSubject, const char* aTopic,
+                      const char16_t* aData) {
   NS_ASSERTION(mAppShell, "appshell service notified before appshell built");
   if (!strcmp(aTopic, "quit-application-forced")) {
     mShuttingDown = true;
-  }
-  else if (!strcmp(aTopic, "profile-change-teardown")) {
+  } else if (!strcmp(aTopic, "profile-change-teardown")) {
     if (!mShuttingDown) {
       EnterLastWindowClosingSurvivalArea();
       CloseAllWindows();
@@ -728,7 +709,7 @@ nsAppStartup::Observe(nsISupports *aSubject,
     if (mPlacesInitCompleteProbe) {
       mPlacesInitCompleteProbe->Trigger();
     }
-#endif //defined(XP_WIN)
+#endif  // defined(XP_WIN)
   } else if (!strcmp(aTopic, "sessionstore-init-started")) {
     StartupTimeline::Record(StartupTimeline::SESSION_RESTORE_INIT);
   } else if (!strcmp(aTopic, "xpcom-shutdown")) {
@@ -737,7 +718,7 @@ nsAppStartup::Observe(nsISupports *aSubject,
     if (mXPCOMShutdownProbe) {
       mXPCOMShutdownProbe->Trigger();
     }
-#endif // defined(XP_WIN)
+#endif  // defined(XP_WIN)
   } else if (!strcmp(aTopic, "quit-application")) {
     StartupTimeline::Record(StartupTimeline::QUIT_APPLICATION);
   } else if (!strcmp(aTopic, "profile-before-change")) {
@@ -750,8 +731,8 @@ nsAppStartup::Observe(nsISupports *aSubject,
 }
 
 NS_IMETHODIMP
-nsAppStartup::GetStartupInfo(JSContext* aCx, JS::MutableHandle<JS::Value> aRetval)
-{
+nsAppStartup::GetStartupInfo(JSContext* aCx,
+                             JS::MutableHandle<JS::Value> aRetval) {
   JS::Rooted<JSObject*> obj(aCx, JS_NewPlainObject(aCx));
 
   aRetval.setObject(*obj);
@@ -765,16 +746,14 @@ nsAppStartup::GetStartupInfo(JSContext* aCx, JS::MutableHandle<JS::Value> aRetva
 
     if (error) {
       Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS,
-        StartupTimeline::PROCESS_CREATION);
+                            StartupTimeline::PROCESS_CREATION);
     }
 
     StartupTimeline::Record(StartupTimeline::PROCESS_CREATION, procTime);
   }
 
   for (int i = StartupTimeline::PROCESS_CREATION;
-       i < StartupTimeline::MAX_EVENT_ID;
-       ++i)
-  {
+       i < StartupTimeline::MAX_EVENT_ID; ++i) {
     StartupTimeline::Event ev = static_cast<StartupTimeline::Event>(i);
     TimeStamp stamp = StartupTimeline::Get(ev);
 
@@ -783,15 +762,16 @@ nsAppStartup::GetStartupInfo(JSContext* aCx, JS::MutableHandle<JS::Value> aRetva
       stamp = procTime;
       MOZ_ASSERT(!stamp.IsNull());
       Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS,
-        StartupTimeline::MAIN);
+                            StartupTimeline::MAIN);
     }
 
     if (!stamp.IsNull()) {
       if (stamp >= procTime) {
-        PRTime prStamp = ComputeAbsoluteTimestamp(stamp)
-          / PR_USEC_PER_MSEC;
-        JS::Rooted<JSObject*> date(aCx, JS::NewDateObject(aCx, JS::TimeClip(prStamp)));
-        JS_DefineProperty(aCx, obj, StartupTimeline::Describe(ev), date, JSPROP_ENUMERATE);
+        PRTime prStamp = ComputeAbsoluteTimestamp(stamp) / PR_USEC_PER_MSEC;
+        JS::Rooted<JSObject*> date(
+            aCx, JS::NewDateObject(aCx, JS::TimeClip(prStamp)));
+        JS_DefineProperty(aCx, obj, StartupTimeline::Describe(ev), date,
+                          JSPROP_ENUMERATE);
       } else {
         Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS, ev);
       }
@@ -802,8 +782,7 @@ nsAppStartup::GetStartupInfo(JSContext* aCx, JS::MutableHandle<JS::Value> aRetva
 }
 
 NS_IMETHODIMP
-nsAppStartup::GetAutomaticSafeModeNecessary(bool *_retval)
-{
+nsAppStartup::GetAutomaticSafeModeNecessary(bool* _retval) {
   NS_ENSURE_ARG_POINTER(_retval);
 
   bool alwaysSafe = false;
@@ -822,8 +801,7 @@ nsAppStartup::GetAutomaticSafeModeNecessary(bool *_retval)
 }
 
 NS_IMETHODIMP
-nsAppStartup::TrackStartupCrashBegin(bool *aIsSafeModeNecessary)
-{
+nsAppStartup::TrackStartupCrashBegin(bool* aIsSafeModeNecessary) {
   const int32_t MAX_TIME_SINCE_STARTUP = 6 * 60 * 60 * 1000;
   const int32_t MAX_STARTUP_BUFFER = 10;
   nsresult rv;
@@ -850,8 +828,7 @@ nsAppStartup::TrackStartupCrashBegin(bool *aIsSafeModeNecessary)
   rv = xr->GetReplacedLockTime(&replacedLockTime);
 
   if (NS_FAILED(rv) || !replacedLockTime) {
-    if (!inSafeMode)
-      Preferences::ClearUser(kPrefRecentCrashes);
+    if (!inSafeMode) Preferences::ClearUser(kPrefRecentCrashes);
     GetAutomaticSafeModeNecessary(aIsSafeModeNecessary);
     return NS_OK;
   }
@@ -863,12 +840,14 @@ nsAppStartup::TrackStartupCrashBegin(bool *aIsSafeModeNecessary)
 
   int32_t recentCrashes = 0;
   Preferences::GetInt(kPrefRecentCrashes, &recentCrashes);
-  mIsSafeModeNecessary = (recentCrashes > maxResumedCrashes && maxResumedCrashes != -1);
+  mIsSafeModeNecessary =
+      (recentCrashes > maxResumedCrashes && maxResumedCrashes != -1);
 
-  // Bug 731613 - Don't check if the last startup was a crash if XRE_PROFILE_PATH is set.  After
-  // profile manager, the profile lock's mod. time has been changed so can't be used on this startup.
-  // After a restart, it's safe to assume the last startup was successful.
-  char *xreProfilePath = PR_GetEnv("XRE_PROFILE_PATH");
+  // Bug 731613 - Don't check if the last startup was a crash if
+  // XRE_PROFILE_PATH is set.  After profile manager, the profile lock's mod.
+  // time has been changed so can't be used on this startup. After a restart,
+  // it's safe to assume the last startup was successful.
+  char* xreProfilePath = PR_GetEnv("XRE_PROFILE_PATH");
   if (xreProfilePath) {
     GetAutomaticSafeModeNecessary(aIsSafeModeNecessary);
     return NS_ERROR_NOT_AVAILABLE;
@@ -882,17 +861,19 @@ nsAppStartup::TrackStartupCrashBegin(bool *aIsSafeModeNecessary)
   int32_t lockSeconds = (int32_t)(replacedLockTime / PR_MSEC_PER_SEC);
 
   // started close enough to good startup so call it good
-  if (lockSeconds <= lastSuccessfulStartup + MAX_STARTUP_BUFFER
-      && lockSeconds >= lastSuccessfulStartup - MAX_STARTUP_BUFFER) {
+  if (lockSeconds <= lastSuccessfulStartup + MAX_STARTUP_BUFFER &&
+      lockSeconds >= lastSuccessfulStartup - MAX_STARTUP_BUFFER) {
     GetAutomaticSafeModeNecessary(aIsSafeModeNecessary);
     return NS_OK;
   }
 
-  // sanity check that the pref set at last success is not greater than the current time
+  // sanity check that the pref set at last success is not greater than the
+  // current time
   if (PR_Now() / PR_USEC_PER_SEC <= lastSuccessfulStartup)
     return NS_ERROR_FAILURE;
 
-  // The last startup was a crash so include it in the count regardless of when it happened.
+  // The last startup was a crash so include it in the count regardless of when
+  // it happened.
   Telemetry::Accumulate(Telemetry::STARTUP_CRASH_DETECTED, true);
 
   if (inSafeMode) {
@@ -907,40 +888,41 @@ nsAppStartup::TrackStartupCrashBegin(bool *aIsSafeModeNecessary)
     recentCrashes++;
     rv = Preferences::SetInt(kPrefRecentCrashes, recentCrashes);
   } else {
-    // Otherwise ignore that crash and all previous since it may not be applicable anymore
-    // and we don't want someone to get stuck in safe mode if their prefs are read-only.
+    // Otherwise ignore that crash and all previous since it may not be
+    // applicable anymore and we don't want someone to get stuck in safe mode if
+    // their prefs are read-only.
     rv = Preferences::ClearUser(kPrefRecentCrashes);
   }
   NS_ENSURE_SUCCESS(rv, rv);
 
   // recalculate since recent crashes count may have changed above
-  mIsSafeModeNecessary = (recentCrashes > maxResumedCrashes && maxResumedCrashes != -1);
+  mIsSafeModeNecessary =
+      (recentCrashes > maxResumedCrashes && maxResumedCrashes != -1);
 
   nsCOMPtr<nsIPrefService> prefs = Preferences::GetService();
-  rv = static_cast<Preferences *>(prefs.get())->SavePrefFileBlocking(); // flush prefs to disk since we are tracking crashes
+  rv = static_cast<Preferences*>(prefs.get())
+           ->SavePrefFileBlocking();  // flush prefs to disk since we are
+                                      // tracking crashes
   NS_ENSURE_SUCCESS(rv, rv);
 
   GetAutomaticSafeModeNecessary(aIsSafeModeNecessary);
   return rv;
 }
 
-static nsresult
-RemoveIncompleteStartupFile()
-{
+static nsresult RemoveIncompleteStartupFile() {
   nsCOMPtr<nsIFile> file;
-  MOZ_TRY(NS_GetSpecialDirectory(NS_APP_USER_PROFILE_LOCAL_50_DIR, getter_AddRefs(file)));
+  MOZ_TRY(NS_GetSpecialDirectory(NS_APP_USER_PROFILE_LOCAL_50_DIR,
+                                 getter_AddRefs(file)));
 
   MOZ_TRY_VAR(file, mozilla::startup::GetIncompleteStartupFile(file));
   return file->Remove(false);
 }
 
 NS_IMETHODIMP
-nsAppStartup::TrackStartupCrashEnd()
-{
+nsAppStartup::TrackStartupCrashEnd() {
   bool inSafeMode = false;
   nsCOMPtr<nsIXULRuntime> xr = do_GetService(XULRUNTIME_SERVICE_CONTRACTID);
-  if (xr)
-    xr->GetInSafeMode(&inSafeMode);
+  if (xr) xr->GetInSafeMode(&inSafeMode);
 
   // return if we already ended or we're restarting into safe mode
   if (mStartupCrashTrackingEnded || (mIsSafeModeNecessary && !inSafeMode))
@@ -953,8 +935,8 @@ nsAppStartup::TrackStartupCrashEnd()
   // detect a recent startup crash.
   Unused << NS_WARN_IF(NS_FAILED(RemoveIncompleteStartupFile()));
 
-  // Use the timestamp of XRE_main as an approximation for the lock file timestamp.
-  // See MAX_STARTUP_BUFFER for the buffer time period.
+  // Use the timestamp of XRE_main as an approximation for the lock file
+  // timestamp. See MAX_STARTUP_BUFFER for the buffer time period.
   TimeStamp mainTime = StartupTimeline::Get(StartupTimeline::MAIN);
   nsresult rv;
 
@@ -964,19 +946,19 @@ nsAppStartup::TrackStartupCrashEnd()
     uint64_t lockFileTime = ComputeAbsoluteTimestamp(mainTime);
 
     rv = Preferences::SetInt(kPrefLastSuccess,
-      (int32_t)(lockFileTime / PR_USEC_PER_SEC));
+                             (int32_t)(lockFileTime / PR_USEC_PER_SEC));
 
     if (NS_FAILED(rv))
       NS_WARNING("Could not set startup crash detection pref.");
   }
 
   if (inSafeMode && mIsSafeModeNecessary) {
-    // On a successful startup in automatic safe mode, allow the user one more crash
-    // in regular mode before returning to safe mode.
+    // On a successful startup in automatic safe mode, allow the user one more
+    // crash in regular mode before returning to safe mode.
     int32_t maxResumedCrashes = 0;
     int32_t prefType;
-    rv = Preferences::GetRootBranch(
-      PrefValueKind::Default)->GetPrefType(kPrefMaxResumedCrashes, &prefType);
+    rv = Preferences::GetRootBranch(PrefValueKind::Default)
+             ->GetPrefType(kPrefMaxResumedCrashes, &prefType);
     NS_ENSURE_SUCCESS(rv, rv);
     if (prefType == nsIPrefBranch::PREF_INT) {
       rv = Preferences::GetInt(kPrefMaxResumedCrashes, &maxResumedCrashes);
@@ -985,7 +967,8 @@ nsAppStartup::TrackStartupCrashEnd()
     rv = Preferences::SetInt(kPrefRecentCrashes, maxResumedCrashes);
     NS_ENSURE_SUCCESS(rv, rv);
   } else if (!inSafeMode) {
-    // clear the count of recent crashes after a succesful startup when not in safe mode
+    // clear the count of recent crashes after a succesful startup when not in
+    // safe mode
     rv = Preferences::ClearUser(kPrefRecentCrashes);
     if (NS_FAILED(rv)) NS_WARNING("Could not clear startup crash count.");
   }
@@ -999,8 +982,7 @@ nsAppStartup::TrackStartupCrashEnd()
 }
 
 NS_IMETHODIMP
-nsAppStartup::RestartInSafeMode(uint32_t aQuitMode)
-{
+nsAppStartup::RestartInSafeMode(uint32_t aQuitMode) {
   PR_SetEnv("MOZ_SAFE_MODE_RESTART=1");
   this->Quit(aQuitMode | nsIAppStartup::eRestart);
 
@@ -1008,8 +990,7 @@ nsAppStartup::RestartInSafeMode(uint32_t aQuitMode)
 }
 
 NS_IMETHODIMP
-nsAppStartup::CreateInstanceWithProfile(nsIToolkitProfile* aProfile)
-{
+nsAppStartup::CreateInstanceWithProfile(nsIToolkitProfile* aProfile) {
   if (NS_WARN_IF(!aProfile)) {
     return NS_ERROR_FAILURE;
   }
@@ -1019,8 +1000,8 @@ nsAppStartup::CreateInstanceWithProfile(nsIToolkitProfile* aProfile)
   }
 
   nsCOMPtr<nsIFile> execPath;
-  nsresult rv = NS_NewLocalFile(gAbsoluteArgv0Path,
-                                true, getter_AddRefs(execPath));
+  nsresult rv =
+      NS_NewLocalFile(gAbsoluteArgv0Path, true, getter_AddRefs(execPath));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -1041,8 +1022,8 @@ nsAppStartup::CreateInstanceWithProfile(nsIToolkitProfile* aProfile)
     return rv;
   }
 
-  const char *args[] = { "-P", profileName.get() };
-  rv = process->Run(false, args, 2);
+  const char* args[] = {"-no-remote", "-P", profileName.get()};
+  rv = process->Run(false, args, 3);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
